#!/usr/bin/python

# Copyright (c) 2019, Evgeniy Krysanov <evgeniy.krysanov@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: bitbucket_pipeline_known_host
short_description: Manages Bitbucket pipeline known hosts
description:
  - Manages Bitbucket pipeline known hosts under the "SSH Keys" menu.
  - The host fingerprint is retrieved automatically, but in case of an error, one can use O(key) field to specify it manually.
author:
  - Evgeniy Krysanov (@catcombo)
extends_documentation_fragment:
  - community.general._bitbucket
  - community.general._attributes
requirements:
  - paramiko
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  repository:
    description:
      - The repository name.
    type: str
    required: true
  workspace:
    description:
      - The repository owner.
      - B(Note:) O(ignore:username) used to be an alias of this option. Since community.general 6.0.0 it is an alias of O(user).
    type: str
    required: true
  name:
    description:
      - The FQDN of the known host.
    type: str
    required: true
  key:
    description:
      - The public key.
    type: str
  state:
    description:
      - Indicates desired state of the record.
    type: str
    required: true
    choices: [absent, present]
notes:
  - Check mode is supported.
"""

EXAMPLES = r"""
- name: Create known hosts from the list
  community.general.bitbucket_pipeline_known_host:
    repository: 'bitbucket-repo'
    workspace: bitbucket_workspace
    name: '{{ item }}'
    state: present
  with_items:
    - bitbucket.org
    - example.com

- name: Remove known host
  community.general.bitbucket_pipeline_known_host:
    repository: bitbucket-repo
    workspace: bitbucket_workspace
    name: bitbucket.org
    state: absent

- name: Specify public key file
  community.general.bitbucket_pipeline_known_host:
    repository: bitbucket-repo
    workspace: bitbucket_workspace
    name: bitbucket.org
    key: '{{lookup("file", "bitbucket.pub") }}'
    state: absent
"""

RETURN = r""" # """

import socket

try:
    import paramiko

    HAS_PARAMIKO = True
except ImportError:
    HAS_PARAMIKO = False

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._bitbucket import BitbucketHelper

error_messages = {
    "invalid_params": "Account or repository was not found",
    "unknown_key_type": "Public key type is unknown",
}

BITBUCKET_API_ENDPOINTS = {
    "known-host-list": f"{BitbucketHelper.BITBUCKET_API_URL}/2.0/repositories/{{workspace}}/{{repo_slug}}/pipelines_config/ssh/known_hosts/",
    "known-host-detail": (
        f"{BitbucketHelper.BITBUCKET_API_URL}/2.0/repositories/{{workspace}}/{{repo_slug}}/pipelines_config/ssh/known_hosts/{{known_host_uuid}}"
    ),
}


def get_existing_known_host(module, bitbucket):
    """
    Search for a host in Bitbucket pipelines known hosts
    with the name specified in module param `name`

    :param module: instance of the :class:`AnsibleModule`
    :param bitbucket: instance of the :class:`BitbucketHelper`
    :return: existing host or None if not found
    :rtype: dict or None

    Return example::

        {
            'type': 'pipeline_known_host',
            'uuid': '{21cc0590-bebe-4fae-8baf-03722704119a7}'
            'hostname': 'bitbucket.org',
            'public_key': {
                'type': 'pipeline_ssh_public_key',
                'md5_fingerprint': 'md5:97:8c:1b:f2:6f:14:6b:4b:3b:ec:aa:46:46:74:7c:40',
                'sha256_fingerprint': 'SHA256:zzXQOXSFBEiUtuE8AikoYKwbHaxvSc0ojez9YXaGp1A',
                'key_type': 'ssh-rsa',
                'key': 'AAAAB3NzaC1yc2EAAAABIwAAAQEAubiN81eDcafrgMeLzaFPsw2kN...seeFVBoGqzHM9yXw=='
            },
        }
    """
    content = {
        "next": BITBUCKET_API_ENDPOINTS["known-host-list"].format(
            workspace=module.params["workspace"],
            repo_slug=module.params["repository"],
        )
    }

    # Look through all response pages in search of hostname we need
    while "next" in content:
        info, content = bitbucket.request(
            api_url=content["next"],
            method="GET",
        )

        if info["status"] == 404:
            module.fail_json(msg="Invalid `repository` or `workspace`.")

        if info["status"] != 200:
            module.fail_json(msg=f"Failed to retrieve list of known hosts: {info}")

        host = next((v for v in content["values"] if v["hostname"] == module.params["name"]), None)

        if host is not None:
            return host

    return None


def get_host_key(module, hostname):
    """
    Fetches public key for specified host

    :param module: instance of the :class:`AnsibleModule`
    :param hostname: host name
    :return: key type and key content
    :rtype: tuple

    Return example::

        (
            'ssh-rsa',
            'AAAAB3NzaC1yc2EAAAABIwAAA...SBne8+seeFVBoGqzHM9yXw==',
        )
    """
    try:
        sock = socket.socket()
        sock.connect((hostname, 22))
    except OSError:
        module.fail_json(msg=f"Error opening socket to {hostname}")

    try:
        trans = paramiko.transport.Transport(sock)
        trans.start_client()
        host_key = trans.get_remote_server_key()
    except paramiko.SSHException:
        module.fail_json(msg=f"SSH error on retrieving {hostname} server key")

    trans.close()
    sock.close()

    key_type = host_key.get_name()
    key = host_key.get_base64()

    return key_type, key


def create_known_host(module, bitbucket):
    hostname = module.params["name"]
    key_param = module.params["key"]

    if key_param is None:
        key_type, key = get_host_key(module, hostname)
    elif " " in key_param:
        key_type, key = key_param.split(" ", 1)
    else:
        module.fail_json(msg=error_messages["unknown_key_type"])

    info, content = bitbucket.request(
        api_url=BITBUCKET_API_ENDPOINTS["known-host-list"].format(
            workspace=module.params["workspace"],
            repo_slug=module.params["repository"],
        ),
        method="POST",
        data={
            "hostname": hostname,
            "public_key": {
                "key_type": key_type,
                "key": key,
            },
        },
    )

    if info["status"] == 404:
        module.fail_json(msg=error_messages["invalid_params"])

    if info["status"] != 201:
        module.fail_json(msg=f"Failed to create known host `{module.params['hostname']}`: {info}")


def delete_known_host(module, bitbucket, known_host_uuid):
    info, content = bitbucket.request(
        api_url=BITBUCKET_API_ENDPOINTS["known-host-detail"].format(
            workspace=module.params["workspace"],
            repo_slug=module.params["repository"],
            known_host_uuid=known_host_uuid,
        ),
        method="DELETE",
    )

    if info["status"] == 404:
        module.fail_json(msg=error_messages["invalid_params"])

    if info["status"] != 204:
        module.fail_json(msg=f"Failed to delete known host `{module.params['name']}`: {info}")


def main():
    argument_spec = BitbucketHelper.bitbucket_argument_spec()
    argument_spec.update(
        repository=dict(type="str", required=True),
        workspace=dict(type="str", required=True),
        name=dict(type="str", required=True),
        key=dict(type="str", no_log=False),
        state=dict(type="str", choices=["present", "absent"], required=True),
    )
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_one_of=BitbucketHelper.bitbucket_required_one_of(),
        required_together=BitbucketHelper.bitbucket_required_together(),
    )

    if (module.params["key"] is None) and (not HAS_PARAMIKO):
        module.fail_json(msg="`paramiko` package not found, please install it.")

    bitbucket = BitbucketHelper(module)

    # Retrieve access token for authorized API requests
    bitbucket.fetch_access_token()

    # Retrieve existing known host
    existing_host = get_existing_known_host(module, bitbucket)
    state = module.params["state"]
    changed = False

    # Create new host in case it doesn't exists
    if not existing_host and (state == "present"):
        if not module.check_mode:
            create_known_host(module, bitbucket)
        changed = True

    # Delete host
    elif existing_host and (state == "absent"):
        if not module.check_mode:
            delete_known_host(module, bitbucket, existing_host["uuid"])
        changed = True

    module.exit_json(changed=changed)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Anvitha Jain (@anvjain) <anvjain@cisco.com>
# Copyright: (c) 2025, Samita Bhattacharjee (@samiib) <samitab@cisco.com>

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: ndo_physical_interface
short_description: Manage Physical Interfaces on Cisco Nexus Dashboard Orchestrator (NDO).
description:
- Manage Physical Interfaces on Cisco Nexus Dashboard Orchestrator (NDO).
- This module is only supported on ND v3.2 (NDO v4.4) and later.
author:
- Anvitha Jain (@anvjain)
options:
  template:
    description:
    - The name of the template.
    - The template must be a fabric resource policy template.
    type: str
    required: true
  name:
    description:
    - The name of the Physical Interface.
    type: str
    aliases: [ physical_interface ]
  uuid:
    description:
    - The UUID of the Physical Interface.
    - This parameter is required when the O(name) needs to be updated.
    type: str
    aliases: [ physical_interface_uuid ]
  description:
    description:
    - The description of the Physical Interface.
    type: str
  nodes:
    description:
    - The node IDs where the Physical Interface policy will be deployed.
    - Each element can either be a single node ID or a range of IDs.
    type: list
    elements: str
  interfaces:
    description:
    - The interface names where the policy will be deployed.
    - The old O(interfaces) will be replaced with the new O(interfaces) during an update.
    type: list
    elements: str
  physical_interface_type:
    description:
    - The type of the interface policy group.
    type: str
    choices: [ physical, breakout ]
  physical_policy_uuid:
    description:
    - The UUID of the Interface Setting Policy.
    - This is only required when creating a new Interface Setting Policy.
    - This parameter is required when O(physical_interface_type) is C(physical).
    - This parameter can be used instead of O(physical_policy).
    type: str
    aliases: [ policy_uuid, interface_policy_uuid , interface_policy_group_uuid, interface_setting_uuid]
  physical_policy:
    description:
    - The interface group policy required for physical Interface Setting Policy.
    - This parameter is required when O(physical_interface_type) is C(physical).
    - This parameter can be used instead of O(physical_policy_uuid).
    type: dict
    suboptions:
      name:
        description:
        - The name of the Interface Setting Policy.
        type: str
      template:
        description:
        - The name of the template in which is referred the Interface Setting Policy.
        type: str
    aliases: [ policy, interface_policy, interface_policy_group, interface_setting ]
  breakout_mode:
    description:
    - Breakout mode enables breaking down an ethernet port into multiple low-speed ports.
    - This parameter is available only when O(physical_interface_type) is C(breakout).
    - The default value is C(4x10G).
    type: str
    choices: [ 4x10G, 4x25G, 4x100G ]
  interface_descriptions:
    description:
    - The interface settings defined in the interface settings policy will be applied to the interfaces on the node IDs configured in C(nodes).
    - This parameter when set to an empty list during an update will clear all the existing interface descriptions.
    - The API will trigger an error when there are duplicate interface IDs in the list.
    type: list
    elements: dict
    suboptions:
      interface_id:
        description:
        - The interface ID.
        type: str
      description:
        description:
        - The description of the interface.
        type: str
  state:
    description:
    - Use C(absent) for removing.
    - Use C(query) for listing an object or multiple objects.
    - Use C(present) for creating or updating.
    type: str
    choices: [ absent, query, present ]
    default: query
notes:
- The O(template) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_template) to create the Tenant template.
- The O(physical_policy) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_interface_setting) to create the Interface Setting Policy.
- The O(physical_policy_uuid) must exist before using this module in your playbook.
  Use M(cisco.mso.ndo_interface_setting) to create the Interface Setting Policy UUID.
seealso:
- module: cisco.mso.ndo_template
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Create a Physical Interface with physical_interface_type set to physical
  cisco.mso.ndo_physical_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    name: ansible_test_physical_interface_physical
    description: "Physical Interface for Ansible Test"
    nodes: [101, 102, 103-105]
    interfaces: "1/1"
    physical_interface_type: physical
    physical_policy: ansible_test_interface_setting_policy_uuid
    state: present
  register: create_physical_interface_type_physical

- name: Create a Physical Interface with physical_interface_type set to breakout
  cisco.mso.ndo_physical_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    name: ansible_test_physical_interface_breakout
    description: "breakout interface for Ansible Test"
    nodes: [101, 102, 103-105]
    interfaces: "1/1"
    physical_interface_type: breakout
    breakout_mode: 4x25G
    interface_descriptions:
      - interface_id: "1/1"
        description: "Interface description for 1/1"
    state: present
  register: create_physical_interface_type_breakout

- name: Query all physical interfaces
  cisco.mso.ndo_physical_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    state: query
  register: query_all

- name: Query a specific Physical Interface with name
  cisco.mso.ndo_physical_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    name: ansible_test_physical_interface_physical
    state: query
  register: query_one_name

- name: Query a specific Physical Interface with UUID
  cisco.mso.ndo_physical_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    uuid: "{{ create_physical_interface_type_breakout.current.uuid }}"
    state: query
  register: query_one_uuid

- name: Delete a Physical Interface with name
  cisco.mso.ndo_physical_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    name: ansible_test_physical_interface_physical
    state: absent

- name: Delete a Physical Interface with UUID
  cisco.mso.ndo_physical_interface:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    template: ansible_test_template
    uuid: "{{ query_one_uuid.current.uuid }}"
    state: absent
"""

RETURN = r"""
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import (
    MSOModule,
    mso_argument_spec,
    format_interface_descriptions,
)
from ansible_collections.cisco.mso.plugins.module_utils.template import MSOTemplate, KVPair
from ansible_collections.cisco.mso.plugins.module_utils.utils import append_update_ops_data
import copy


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        dict(
            template=dict(type="str", required=True),
            name=dict(type="str", aliases=["physical_interface"]),
            uuid=dict(type="str", aliases=["physical_interface_uuid"]),
            description=dict(type="str"),
            nodes=dict(type="list", elements="str"),
            interfaces=dict(type="list", elements="str"),
            physical_interface_type=dict(type="str", choices=["physical", "breakout"]),
            physical_policy_uuid=dict(type="str", aliases=["policy_uuid", "interface_policy_uuid", "interface_policy_group_uuid", "interface_setting_uuid"]),
            physical_policy=dict(
                type="dict",
                options=dict(
                    name=dict(type="str"),
                    template=dict(type="str"),
                ),
                aliases=["policy", "interface_policy", "interface_policy_group", "interface_setting"],
            ),
            breakout_mode=dict(type="str", choices=["4x10G", "4x25G", "4x100G"]),
            interface_descriptions=dict(
                type="list",
                elements="dict",
                options=dict(
                    interface_id=dict(type="str"),
                    description=dict(type="str"),
                ),
            ),
            state=dict(type="str", default="query", choices=["absent", "query", "present"]),
        )
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["name", "uuid"], True],
            ["state", "absent", ["name", "uuid"], True],
        ],
        mutually_exclusive=[("physical_policy", "breakout_mode"), ("physical_policy", "physical_policy_uuid")],
    )

    mso = MSOModule(module)

    template = module.params.get("template")
    name = module.params.get("name")
    uuid = module.params.get("uuid")
    description = module.params.get("description")
    nodes = module.params.get("nodes")
    interfaces = module.params.get("interfaces")
    if interfaces:
        interfaces = ",".join(interfaces)
    physical_interface_type = module.params.get("physical_interface_type")
    physical_policy_uuid = module.params.get("physical_policy_uuid")
    physical_policy = module.params.get("physical_policy")
    breakout_mode = module.params.get("breakout_mode")
    interface_descriptions = module.params.get("interface_descriptions")
    state = module.params.get("state")

    ops = []
    match = None
    physical_interface_attrs_path = None

    mso_template = MSOTemplate(mso, "fabric_resource", template)
    mso_template.validate_template("fabricResource")
    object_description = "Physical Interface Profile"
    path = "/fabricResourceTemplate/template/interfaceProfiles"

    existing_physical_interfaces = mso_template.template.get("fabricResourceTemplate", {}).get("template", {}).get("interfaceProfiles") or []

    if name or uuid:
        match = mso_template.get_object_by_key_value_pairs(
            object_description,
            existing_physical_interfaces,
            [KVPair("uuid", uuid) if uuid else KVPair("name", name)],
        )
        if match:
            physical_interface_attrs_path = "{0}/{1}".format(path, match.index)
            mso.existing = mso.previous = copy.deepcopy(match.details)
    else:
        mso.existing = mso.previous = existing_physical_interfaces

    if state == "present":
        if uuid and not mso.existing:
            mso.fail_json(msg="{0} with the UUID: '{1}' not found".format(object_description, uuid))

        if physical_policy and not physical_policy_uuid:
            fabric_policy_template = MSOTemplate(mso, "fabric_policy", physical_policy.get("template"))
            fabric_policy_template.validate_template("fabricPolicy")
            physical_policy_uuid = fabric_policy_template.get_interface_policy_group_uuid(physical_policy.get("name"))

        mso_values = dict(
            name=name,
            description=description,
            nodes=nodes,
            interfaces=interfaces,
            policyGroupType=physical_interface_type,
        )

        if physical_interface_type == "physical" and physical_policy_uuid:
            mso_values["policy"] = physical_policy_uuid

        if physical_interface_type == "breakout" and breakout_mode:
            mso_values["breakoutMode"] = breakout_mode

        if interface_descriptions:
            mso_values["interfaceDescriptions"] = format_interface_descriptions(mso, interface_descriptions, "")

        if mso.existing and match:
            proposed_payload = copy.deepcopy(mso.existing)
            mso_values_remove = list()

            if physical_interface_type and match.details.get("policyGroupType") != physical_interface_type:
                mso.fail_json(msg="ERROR: Physical Interface type cannot be changed.")

            if interface_descriptions == [] and proposed_payload.get("interfaceDescriptions"):
                mso_values_remove.append("interfaceDescriptions")

            append_update_ops_data(ops, match.details, physical_interface_attrs_path, mso_values, mso_values_remove)
            mso.sanitize(match.details, collate=True)

        else:
            if not nodes:
                mso.fail_json(msg=("ERROR: Missing 'nodes' for creating a Physical Interface."))

            if not physical_interface_type:
                mso.fail_json(msg=("ERROR: Missing Physical Interface type for creating a Physical Interface."))

            mso.sanitize(mso_values)
            ops.append(dict(op="add", path="{0}/-".format(path), value=mso.sent))

    elif state == "absent":
        if match:
            ops.append(dict(op="remove", path=physical_interface_attrs_path))

    if not module.check_mode and ops:
        response = mso.request(mso_template.template_path, method="PATCH", data=ops)
        physical_interfaces = response.get("fabricResourceTemplate", {}).get("template", {}).get("interfaceProfiles") or []
        match = mso_template.get_object_by_key_value_pairs(object_description, physical_interfaces, [KVPair("uuid", uuid) if uuid else KVPair("name", name)])
        if match:
            mso.existing = match.details  # When the state is present
        else:
            mso.existing = {}  # When the state is absent
    elif module.check_mode and state != "query":  # When the state is present/absent with check mode
        mso.existing = mso.proposed if state == "present" else {}

    mso.exit_json()


if __name__ == "__main__":
    main()



# Copyright (c) 2022 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this doc fragment is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations


class ModuleDocFragment:

    DOCUMENTATION = r"""
options:
  icann_only:
    description:
      - This controls whether only entries from the ICANN section of the Public Suffix List are used, or also entries from
        the Private section. For example, C(.co.uk) is in the ICANN section, but C(github.io) is in the Private section.
    type: boolean
    default: false
"""

    PUBLIC_SUFFIX = r"""
options:
  keep_unknown_suffix:
    description:
      - This treats unknown TLDs as valid public suffixes. So for example the public suffix
        of C(example.tlddoesnotexist) is C(.tlddoesnotexist) if this is V(true). If set to
        V(false), it will return an empty string in this case.
      - This option corresponds to whether the global wildcard rule C(*) in the Public
        Suffix List is used or not.
    type: boolean
    default: true
"""

    REGISTERABLE_DOMAIN = r"""
options:
  only_if_registerable:
    description:
      - This controls the behavior in case there is no label in front of the public suffix.
        This is the case if the DNS name itself is a public suffix.
      - If set to V(false), in this case the public suffix is treated as a registrable domain.
      - If set to V(true) (default), the registrable domain of a public suffix is interpreted as an
        empty string.
    type: boolean
    default: true
  keep_unknown_suffix:
    description:
      - This treats unknown TLDs as valid public suffixes. So for example the public suffix of
        C(example.tlddoesnotexist) is C(.tlddoesnotexist) if this is V(true), and hence the
        registrable domain of C(www.example.tlddoesnotexist) is C(example.tlddoesnotexist).
        If set to V(false), the registrable domain of C(www.example.tlddoesnotexist) is
        C(tlddoesnotexist).
      - This option corresponds to whether the global wildcard rule C(*) in the Public Suffix List
        is used or not.
    type: boolean
    default: true
"""

    GET = r"""
options:
  normalize_result:
    description:
      - This controls whether the result is reconstructed from the normalized name used during lookup. During normalization,
        ulabels are converted to alabels, and every label is converted to lowercase. For example, the ulabel C(ëçãmplê) is
        converted to C(xn--mpl-llatwb) (puny-code), and C(Example.COM) is converted to C(example.com).
    type: boolean
    default: false
"""



#!/usr/bin/python
# Copyright (c) 2025, Alexei Znamensky <russoz@gmail.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: django_dumpdata
author:
  - Alexei Znamensky (@russoz)
short_description: Wrapper for C(django-admin dumpdata)
version_added: 11.3.0
description:
  - This module is a wrapper for the execution of C(django-admin dumpdata).
extends_documentation_fragment:
  - community.general._attributes
  - community.general._django
  - community.general._django.database
  - community.general._django.data
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
options:
  all:
    description: Dump all records, including those which might otherwise be filtered or modified by a custom manager.
    type: bool
  indent:
    description:
      - Indentation size for the output.
      - Default is not to indent, so the output is generated in one single line.
    type: int
  natural_foreign:
    description: Use natural keys when serializing for foreign keys.
    type: bool
  natural_primary:
    description: Omit primary keys when serializing.
    type: bool
  primary_keys:
    description:
      - List of primary keys to include in the dump.
      - Only available when dumping one single model.
    type: list
    elements: str
    aliases: ["pks"]
  fixture:
    description:
      - Path to the output file.
      - The fixture filename may end with V(.bz2), V(.gz), V(.lzma) or V(.xz), in which case the corresponding
        compression format is used.
      - This corresponds to the C(--output) parameter for the C(django-admin dumpdata) command.
    type: path
    aliases: [output]
    required: true
  apps_models:
    description:
      - Dump only the applications and models listed in the dump.
      - Format must be either V(app_label) or V(app_label.ModelName).
      - If not passed, all applications and models are to be dumped.
    type: list
    elements: str
"""

EXAMPLES = r"""
- name: Dump all data
  community.general.django_dumpdata:
    settings: myproject.settings
    fixture: /tmp/mydata.json

- name: Dump data excluding certain apps, into a compressed JSON file
  community.general.django_dumpdata:
    settings: myproject.settings
    database: myotherdb
    excludes:
      - auth
      - contenttypes
    fixture: /tmp/mydata.json.gz
"""

RETURN = r"""
run_info:
  description: Command-line execution information.
  type: dict
  returned: success and O(verbosity) >= 3
version:
  description: Version of Django.
  type: str
  returned: always
  sample: 5.1.2
"""

from ansible_collections.community.general.plugins.module_utils._django import DjangoModuleHelper


class DjangoDumpData(DjangoModuleHelper):
    module = dict(
        argument_spec=dict(
            all=dict(type="bool"),
            indent=dict(type="int"),
            natural_foreign=dict(type="bool"),
            natural_primary=dict(type="bool"),
            primary_keys=dict(type="list", elements="str", aliases=["pks"], no_log=False),
            # the underlying vardict does not allow the name "output"
            fixture=dict(type="path", required=True, aliases=["output"]),
            apps_models=dict(type="list", elements="str"),
        ),
        supports_check_mode=False,
    )
    django_admin_cmd = "dumpdata"
    django_admin_arg_order = (
        "all format indent excludes database_dash natural_foreign natural_primary primary_keys fixture apps_models"
    )
    _django_args = ["data", "database_dash"]

    def __init_module__(self):
        self.vars.set("database_dash", self.vars.database, output=False)


def main():
    DjangoDumpData.execute()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Dag Wieers <dag@wieers.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: imc_rest
short_description: Manage Cisco IMC hardware through its REST API
description:
  - Provides direct access to the Cisco IMC REST API.
  - Perform any configuration changes and actions that the Cisco IMC supports.
  - More information about the IMC REST API is available from
    U(http://www.cisco.com/c/en/us/td/docs/unified_computing/ucs/c/sw/api/3_0/b_Cisco_IMC_api_301.html).
author:
  - Dag Wieers (@dagwieers)
requirements:
  - lxml
  - xmljson >= 0.1.8
extends_documentation_fragment:
  - community.general._attributes
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  hostname:
    description:
      - IP Address or hostname of Cisco IMC, resolvable by Ansible control host.
    required: true
    aliases: [host, ip]
    type: str
  username:
    description:
      - Username used to login to the switch.
    default: admin
    aliases: [user]
    type: str
  password:
    description:
      - The password to use for authentication.
    default: password
    type: str
  path:
    description:
      - Name of the absolute path of the filename that includes the body of the http request being sent to the Cisco IMC REST
        API.
      - Parameter O(path) is mutual exclusive with parameter O(content).
    aliases: ['src', 'config_file']
    type: path
  content:
    description:
      - When used instead of O(path), sets the content of the API requests directly.
      - This may be convenient to template simple requests, for anything complex use the M(ansible.builtin.template) module.
      - You can collate multiple IMC XML fragments and they are processed sequentially in a single stream, the Cisco IMC output
        is subsequently merged.
      - Parameter O(content) is mutual exclusive with parameter O(path).
    type: str
  protocol:
    description:
      - Connection protocol to use.
    default: https
    choices: [http, https]
    type: str
  timeout:
    description:
      - The socket level timeout in seconds.
      - This is the time that every single connection (every fragment) can spend. If this O(timeout) is reached, the module
        fails with a C(Connection failure) indicating that C(The read operation timed out).
    default: 60
    type: int
  validate_certs:
    description:
      - If V(false), SSL certificates are not validated.
      - This should only set to V(false) used on personally controlled sites using self-signed certificates.
    type: bool
    default: true
notes:
  - The XML fragments do not need an authentication cookie, this is injected by the module automatically.
  - The Cisco IMC XML output is being translated to JSON using the Cobra convention.
  - Any configConfMo change requested has a return status of C(modified), even if there was no actual change from the previous
    configuration. As a result, this module always reports a change on subsequent runs. In case this behaviour is fixed in
    a future update to Cisco IMC, this module is meant to automatically adapt.
  - If you get a C(Connection failure) related to C(The read operation timed out) increase the O(timeout) parameter. Some
    XML fragments can take longer than the default timeout.
  - More information about the IMC REST API is available from
    U(http://www.cisco.com/c/en/us/td/docs/unified_computing/ucs/c/sw/api/3_0/b_Cisco_IMC_api_301.html).
"""

EXAMPLES = r"""
- name: Power down server
  community.general.imc_rest:
    hostname: '{{ imc_hostname }}'
    username: '{{ imc_username }}'
    password: '{{ imc_password }}'
    validate_certs: false # only do this when you trust the network!
    content: |
      <configConfMo><inConfig>
        <computeRackUnit dn="sys/rack-unit-1" adminPower="down"/>
      </inConfig></configConfMo>
  delegate_to: localhost

- name: Configure IMC using multiple XML fragments
  community.general.imc_rest:
    hostname: '{{ imc_hostname }}'
    username: '{{ imc_username }}'
    password: '{{ imc_password }}'
    validate_certs: false # only do this when you trust the network!
    timeout: 120
    content: |
      <!-- Configure Serial-on-LAN -->
      <configConfMo><inConfig>
        <solIf dn="sys/rack-unit-1/sol-if" adminState="enable" speed=="115200" comport="com0"/>
      </inConfig></configConfMo>

      <!-- Configure Console Redirection -->
      <configConfMo><inConfig>
        <biosVfConsoleRedirection dn="sys/rack-unit-1/bios/bios-settings/Console-redirection"
          vpBaudRate="115200"
          vpConsoleRedirection="com-0"
          vpFlowControl="none"
          vpTerminalType="vt100"
          vpPuttyKeyPad="LINUX"
          vpRedirectionAfterPOST="Always Enable"/>
      </inConfig></configConfMo>
  delegate_to: localhost

- name: Enable PXE boot and power-cycle server
  community.general.imc_rest:
    hostname: '{{ imc_hostname }}'
    username: '{{ imc_username }}'
    password: '{{ imc_password }}'
    validate_certs: false # only do this when you trust the network!
    content: |
      <!-- Configure PXE boot -->
      <configConfMo><inConfig>
        <lsbootLan dn="sys/rack-unit-1/boot-policy/lan-read-only" access="read-only" order="1" prot="pxe" type="lan"/>
      </inConfig></configConfMo>

      <!-- Power cycle server -->
      <configConfMo><inConfig>
        <computeRackUnit dn="sys/rack-unit-1" adminPower="cycle-immediate"/>
      </inConfig></configConfMo>
  delegate_to: localhost

- name: Reconfigure IMC to boot from storage
  community.general.imc_rest:
    hostname: '{{ imc_host }}'
    username: '{{ imc_username }}'
    password: '{{ imc_password }}'
    validate_certs: false # only do this when you trust the network!
    content: |
      <configConfMo><inConfig>
        <lsbootStorage dn="sys/rack-unit-1/boot-policy/storage-read-write" access="read-write" order="1" type="storage"/>
      </inConfig></configConfMo>
  delegate_to: localhost

- name: Add customer description to server
  community.general.imc_rest:
    hostname: '{{ imc_host }}'
    username: '{{ imc_username }}'
    password: '{{ imc_password }}'
    validate_certs: false # only do this when you trust the network!
    content: |
      <configConfMo><inConfig>
        <computeRackUnit dn="sys/rack-unit-1" usrLbl="Customer Lab - POD{{ pod_id }} - {{ inventory_hostname_short }}"/>
      </inConfig></configConfMo>
    delegate_to: localhost

- name: Disable HTTP and increase session timeout to max value 10800 secs
  community.general.imc_rest:
    hostname: '{{ imc_host }}'
    username: '{{ imc_username }}'
    password: '{{ imc_password }}'
    validate_certs: false # only do this when you trust the network!
    timeout: 120
    content: |
      <configConfMo><inConfig>
        <commHttp dn="sys/svc-ext/http-svc" adminState="disabled"/>
      </inConfig></configConfMo>

      <configConfMo><inConfig>
        <commHttps dn="sys/svc-ext/https-svc" adminState="enabled" sessionTimeout="10800"/>
      </inConfig></configConfMo>
    delegate_to: localhost
"""

RETURN = r"""
aaLogin:
  description: Cisco IMC XML output for the login, translated to JSON using Cobra convention.
  returned: success
  type: dict
  sample: |
    "attributes": {
        "cookie": "",
        "outCookie": "1498902428/9de6dc36-417c-157c-106c-139efe2dc02a",
        "outPriv": "admin",
        "outRefreshPeriod": "600",
        "outSessionId": "114",
        "outVersion": "2.0(13e)",
        "response": "yes"
    }
configConfMo:
  description: Cisco IMC XML output for any configConfMo XML fragments, translated to JSON using Cobra convention.
  returned: success
  type: dict
  sample: |
elapsed:
  description: Elapsed time in seconds.
  returned: always
  type: int
  sample: 31
response:
  description: HTTP response message, including content length.
  returned: always
  type: str
  sample: OK (729 bytes)
status:
  description: The HTTP response status code.
  returned: always
  type: dict
  sample: 200
error:
  description: Cisco IMC XML error output for last request, translated to JSON using Cobra convention.
  returned: failed
  type: dict
  sample: |
    "attributes": {
        "cookie": "",
        "errorCode": "ERR-xml-parse-error",
        "errorDescr": "XML PARSING ERROR: Element 'computeRackUnit', attribute 'admin_Power': The attribute 'admin_Power' is not allowed. ",
        "invocationResult": "594",
        "response": "yes"
    }
error_code:
  description: Cisco IMC error code.
  returned: failed
  type: str
  sample: ERR-xml-parse-error
error_text:
  description: Cisco IMC error message.
  returned: failed
  type: str
  sample: |
    XML PARSING ERROR: Element 'computeRackUnit', attribute 'admin_Power': The attribute 'admin_Power' is not allowed.
input:
  description: RAW XML input sent to the Cisco IMC, causing the error.
  returned: failed
  type: str
  sample: |
    <configConfMo><inConfig><computeRackUnit dn="sys/rack-unit-1" admin_Power="down"/></inConfig></configConfMo>
output:
  description: RAW XML output received from the Cisco IMC, with error details.
  returned: failed
  type: str
  sample: >
    <error cookie=""
      response="yes"
      errorCode="ERR-xml-parse-error"
      invocationResult="594"
      errorDescr="XML PARSING ERROR: Element 'computeRackUnit', attribute 'admin_Power': The attribute 'admin_Power' is not allowed.\n" />
"""

import os
import traceback
from itertools import zip_longest

LXML_ETREE_IMP_ERR = None
try:
    import lxml.etree

    HAS_LXML_ETREE = True
except ImportError:
    LXML_ETREE_IMP_ERR = traceback.format_exc()
    HAS_LXML_ETREE = False

XMLJSON_COBRA_IMP_ERR = None
try:
    from xmljson import cobra

    HAS_XMLJSON_COBRA = True
except ImportError:
    XMLJSON_COBRA_IMP_ERR = traceback.format_exc()
    HAS_XMLJSON_COBRA = False

from ansible.module_utils.basic import AnsibleModule, missing_required_lib
from ansible.module_utils.urls import fetch_url

from ansible_collections.community.general.plugins.module_utils._datetime import (
    now,
)


def imc_response(module, rawoutput, rawinput=""):
    """Handle IMC returned data"""
    xmloutput = lxml.etree.fromstring(rawoutput)
    result = cobra.data(xmloutput)

    # Handle errors
    if xmloutput.get("errorCode") and xmloutput.get("errorDescr"):
        if rawinput:
            result["input"] = rawinput
        result["output"] = rawoutput
        result["error_code"] = xmloutput.get("errorCode")
        result["error_text"] = xmloutput.get("errorDescr")
        module.fail_json(msg=f"Request failed: {result['error_text']}", **result)

    return result


def logout(module, url, cookie, timeout):
    """Perform a logout, if needed"""
    data = f'<aaaLogout cookie="{cookie}" inCookie="{cookie}"/>'
    resp, auth = fetch_url(module, url, data=data, method="POST", timeout=timeout)


def merge(one, two):
    """Merge two complex nested datastructures into one"""
    if isinstance(one, dict) and isinstance(two, dict):
        copy = dict(one)
        copy.update({key: merge(one.get(key, None), two[key]) for key in two})
        return copy

    elif isinstance(one, list) and isinstance(two, list):
        return [merge(alpha, beta) for (alpha, beta) in zip_longest(one, two)]

    return one if two is None else two


def main():
    module = AnsibleModule(
        argument_spec=dict(
            hostname=dict(type="str", required=True, aliases=["host", "ip"]),
            username=dict(type="str", default="admin", aliases=["user"]),
            password=dict(type="str", default="password", no_log=True),
            content=dict(type="str"),
            path=dict(type="path", aliases=["config_file", "src"]),
            protocol=dict(type="str", default="https", choices=["http", "https"]),
            timeout=dict(type="int", default=60),
            validate_certs=dict(type="bool", default=True),
        ),
        supports_check_mode=True,
        mutually_exclusive=[["content", "path"]],
    )

    if not HAS_LXML_ETREE:
        module.fail_json(msg=missing_required_lib("lxml"), exception=LXML_ETREE_IMP_ERR)

    if not HAS_XMLJSON_COBRA:
        module.fail_json(msg=missing_required_lib("xmljson >= 0.1.8"), exception=XMLJSON_COBRA_IMP_ERR)

    hostname = module.params["hostname"]
    username = module.params["username"]
    password = module.params["password"]

    content = module.params["content"]
    path = module.params["path"]

    protocol = module.params["protocol"]
    timeout = module.params["timeout"]

    result = dict(
        failed=False,
        changed=False,
    )

    # Report missing file
    file_exists = False
    if path:
        if os.path.isfile(path):
            file_exists = True
        else:
            module.fail_json(msg=f"Cannot find/access path:\n{path}")

    start = now()

    # Perform login first
    url = f"{protocol}://{hostname}/nuova"
    data = f'<aaaLogin inName="{username}" inPassword="{password}"/>'
    resp, auth = fetch_url(module, url, data=data, method="POST", timeout=timeout)
    if resp is None or auth["status"] != 200:
        result["elapsed"] = (now() - start).seconds
        module.fail_json(msg=f"Task failed with error {auth['status']}: {auth['msg']}", **result)
    result.update(imc_response(module, resp.read()))

    # Store cookie for future requests
    cookie = ""
    try:
        cookie = result["aaaLogin"]["attributes"]["outCookie"]
    except Exception:
        module.fail_json(msg="Could not find cookie in output", **result)

    try:
        # Prepare request data
        if content:
            rawdata = content.replace("\n", "")
        elif file_exists:
            with open(path) as config_object:
                rawdata = config_object.read().replace("\n", "")

        # Wrap the XML documents in a <root> element
        xmldata = lxml.etree.fromstring(f"<root>{rawdata}</root>")

        # Handle each XML document separately in the same session
        for xmldoc in list(xmldata):
            if xmldoc.tag is lxml.etree.Comment:
                continue
            # Add cookie to XML
            xmldoc.set("cookie", cookie)
            data = lxml.etree.tostring(xmldoc)

            # Perform actual request
            resp, info = fetch_url(module, url, data=data, method="POST", timeout=timeout)
            if resp is None or info["status"] != 200:
                result["elapsed"] = (now() - start).seconds
                module.fail_json(msg=f"Task failed with error {info['status']}: {info['msg']}", **result)

            # Merge results with previous results
            rawoutput = resp.read()
            result = merge(result, imc_response(module, rawoutput, rawinput=data))
            result["response"] = info["msg"]
            result["status"] = info["status"]

            # Check for any changes
            # NOTE: Unfortunately IMC API always report status as 'modified'
            xmloutput = lxml.etree.fromstring(rawoutput)
            results = xmloutput.xpath("/configConfMo/outConfig/*/@status")
            result["changed"] = "modified" in results

        # Report success
        result["elapsed"] = (now() - start).seconds
        module.exit_json(**result)
    finally:
        logout(module, url, cookie, timeout)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2023 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_fc_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
---
module: nxos_fc_interfaces
short_description: Fc Interfaces resource module
description: This module manages the interface attributes of NX-OS fc interfaces.
version_added: 5.2.0
author: Suhas Bharadwaj (@srbharadwaj)
notes:
  - Tested against NXOS 9.3(2) on Cisco MDS Switches
options:
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the NX-OS
        device by executing the command B(show running-config interface)
      - The state I(parsed) reads the configuration from C(running_config)
        option and transforms it into Ansible structured data as per the
        resource module's argspec and the value is then returned in the
        I(parsed) key within the result.
    type: str
  config:
    description: A dictionary of interface options
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - Full name of interface, e.g. fc1/1, fc18/48
        type: str
        required: true
      description:
        description:
          - Interface description.
        type: str
      enabled:
        description:
          - Administrative state of the interface. Set the value to C(true) to
            administratively enable the interface or C(true) to disable it
        type: bool
      speed:
        description:
          - Interface link speed.
        choices:
          - 'auto'
          - '1000'
          - '2000'
          - '4000'
          - '8000'
          - '10000'
          - '16000'
          - '32000'
          - '64000'
          - 'auto max 2000'
          - 'auto max 4000'
          - 'auto max 8000'
          - 'auto max 16000'
          - 'auto max 32000'
          - 'auto max 64000'
        type: str
      mode:
        description:
          - Port mode of the fc interface
        choices:
          - 'auto'
          - 'E'
          - 'F'
          - 'Fx'
          - 'NP'
          - 'SD'
        type: str
      trunk_mode:
        description:
          - Trunk mode of the fc interface
        choices:
          - 'auto'
          - 'on'
          - 'off'
        type: str
      analytics:
        description:
          - Analytics type on the fc interface
        choices:
          - 'fc-scsi'
          - 'fc-nvme'
          - 'fc-all'
        type: str
  state:
    description:
      - The state of the configuration after module completion
    type: str
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - gathered
      - rendered
      - parsed
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# switch# show running-config interface all
# interface fc18/10
#     analytics type fc-nvme
#     switchport speed auto max 16000
#     switchport mode auto
#     switchport description $
#     switchport trunk mode on
#     shutdown

- name: Merge provided configuration with device configuration
  cisco.nxos.nxos_fc_interfaces:
    config:
      - name: fc18/10
        analytics: fc-scsi
    state: merged

# Task Output
# -----------
#
# before:
# - name: fc18/10
#   speed: auto max 16000
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: $
#   analytics: fc-nvme
# commands:
# - interface fc18/10
# - analytics type fc-scsi
# after:
# - name: fc18/10
#   speed: auto max 16000
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: $
#   analytics: fc-all

# After state:
# ------------
#
# switch# show running-config interface all
# interface fc18/10
#     analytics type fc-scsi
#     analytics type fc-nvme
#     switchport speed auto max 16000
#     switchport mode auto
#     switchport description $
#     switchport trunk mode on
#     shutdown

# Using replaced

# Before state:
# -------------
#
# switch# show running-config interface all
# interface fc18/12
#     analytics type fc-scsi
#     analytics type fc-nvme
#     switchport speed auto max 64000
#     switchport mode auto
#     switchport description 1
#     switchport trunk mode on
#     no shutdown

- name: Replaces device configuration of listed interfaces with provided configuration
  cisco.nxos.nxos_fc_interfaces:
    config:
      - name: fc18/12
        speed: auto max 64000
        mode: auto
        trunk_mode: "on"
        enabled: true
        description: 1
        analytics: fc-scsi
    state: replaced

# Task Output
# -----------
#
# before:
# - name: fc18/12
#   speed: auto max 64000
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: 1
#   analytics: fc-all
# commands:
# - interface fc18/12
# - no analytics type fc-all
# - analytics type fc-scsi
# after:
# - name: fc18/12
#   speed: auto max 64000
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: 1
#   analytics: fc-scsi

# After state:
# ------------
#
# switch# show running-config interface all
# interface fc18/12
#     analytics type fc-scsi
#     switchport speed auto max 64000
#     switchport mode auto
#     switchport description 1
#     switchport trunk mode on
#     no shutdown


# Using deleted

# Before state:
# -------------
#
# switch# show running-config interface all
# interface fc1/2
#     switchport speed 1000
#     switchport mode E
#     no switchport description
#     switchport trunk mode off
#     no shutdown

- name: Delete or return interface parameters to default settings
  cisco.nxos.nxos_fc_interfaces:
    config:
      - name: fc1/2
    state: deleted

# Task Output
# -----------
#
# before:
# - name: fc1/2
#   speed: 1000
#   mode: E
#   trunk_mode: off
#   enabled: true
# commands:
# - interface fc1/2
# - no switchport speed 1000
# - no switchport mode E
# - switchport trunk mode on
# - shutdown
# after:
# - name: fc1/2
#   speed: auto
#   mode: auto
#   trunk_mode: on
#   enabled: true

# After state:
# ------------
#
# switch# show running-config interface all
# interface fc1/2
#     switchport speed auto
#     switchport mode auto
#     no switchport description
#     switchport trunk mode on
#     shutdown

# Using overridden

# Before state:
# -------------
#
# switch# show running-config interface all
# interface fc18/12
#     analytics type fc-scsi
#     analytics type fc-nvme
#     switchport speed auto max 64000
#     switchport mode auto
#     switchport description 1
#     switchport trunk mode on
#     no shutdown
# interface fc18/13
#     analytics type fc-scsi
#     analytics type fc-nvme
#     switchport speed auto max 64000
#     switchport mode auto
#     switchport description 1
#     switchport trunk mode on
#     no shutdown

- name: Replaces device configuration of listed interfaces with provided configuration
  cisco.nxos.nxos_fc_interfaces:
    config:
      - name: fc18/12
        speed: auto max 64000
        mode: auto
        trunk_mode: "on"
        enabled: true
        description: 1
        analytics: fc-scsi
    state: overridden

# Task Output
# -----------
#
# before:
# - name: fc18/12
#   speed: auto max 64000
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: 1
#   analytics: fc-all
# - name: fc18/13
#   speed: auto max 64000
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: 1
#   analytics: fc-all
# commands:
# - interface fc18/12
#   no analytics type fc-all
#   analytics type fc-scsi
# - interface fc18/13
#   no switchport description
#   no switchport speed auto max 64000
#   no switchport mode auto
#   switchport trunk mode on
#   shutdown
# after:
# - name: fc18/12
#   speed: auto max 64000
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: 1
#   analytics: fc-scsi
# - name: fc18/13
#   speed: auto max 64000
#   mode: auto
#   trunk_mode: on
#   enabled: true

# After state:
# ------------
#
# switch# show running-config interface all
# interface fc18/12
#     analytics type fc-scsi
#     switchport speed auto max 64000
#     switchport mode auto
#     switchport description 1
#     switchport trunk mode on
#     no shutdown
# interface fc18/13
#     switchport mode auto
#     switchport trunk mode on
#     shutdown

# Using rendered

- name: Use rendered state to convert task input to device specific commands
  cisco.nxos.nxos_fc_interfaces:
    config:
      - name: fc1/1
        speed: auto
        mode: auto
        trunk_mode: "on"
        enabled: true
        description: This is a sample line
      - name: fc1/2
        speed: 1000
        mode: E
        trunk_mode: "off"
        enabled: true
        state: rendered

# Task Output
# -----------
#
# rendered:
# interface fc1/1
#     switchport speed auto
#     switchport mode auto
#     switchport description This is a sample line
#     switchport trunk mode on
#     no shutdown
#
# interface fc1/2
#     switchport speed 1000
#     switchport mode E
#     no switchport description
#     switchport trunk mode off
#     no shutdown

# Using parsed

# parsed.cfg
# ------------
#
# interface fc1/1
#     switchport speed auto
#     switchport mode auto
#     switchport description This is a sample line
#     switchport trunk mode on
#     no shutdown
#
# interface fc1/2
#     switchport speed 1000
#     switchport mode E
#     no switchport description
#     switchport trunk mode off
#     no shutdown

- name: Use parsed state to convert externally supplied config to structured format
  cisco.nxos.nxos_fc_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output
# -----------
#
#  parsed:
# - name: fc1/1
#   speed: auto
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: This is a sample line
# - name: fc1/2
#   speed: 1000
#   mode: E
#   trunk_mode: off
#   enabled: true

# Using gathered

# Before state:
# -------------
#
# switch# show running-config | section interface
# interface fc1/1
#     switchport speed auto
#     switchport mode auto
#     switchport description This is a sample line
#     switchport trunk mode on
#     no shutdown
#
# interface fc1/2
#     switchport speed 1000
#     switchport mode E
#     no switchport description
#     switchport trunk mode off
#     no shutdown
#
- name: Gather interfaces facts from the device using nxos_fc_interfaces
  cisco.nxos.nxos_fc_interfaces:
    state: gathered
#
# Task output
# -----------
#
# - name: fc1/1
#   speed: auto
#   mode: auto
#   trunk_mode: on
#   enabled: true
#   description: This is a sample line
# - name: fc1/2
#   speed: 1000
#   mode: E
#   trunk_mode: off
#   enabled: true
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - interface fc1/1
    - description sample description
    - shutdown
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - interface fc1/1
    - description sample description
    - shutdown
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.fc_interfaces.fc_interfaces import (
    Fc_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.fc_interfaces.fc_interfaces import (
    Fc_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Fc_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Fc_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# (c) 2022 Red Hat Inc.
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from ansible.module_utils.common.text.converters import to_native


class AnsibleAWSError(Exception):
    def __str__(self):
        if self.exception and self.message:
            return f"{self.message}: {to_native(self.exception)}"

        return super().__str__()

    def __init__(self, message=None, exception=None, **kwargs):
        if not message and not exception:
            super().__init__()
        elif not message:
            super().__init__(exception)
        else:
            super().__init__(message)

        self.exception = exception
        self.message = message

        # In places where passing more information to module.fail_json would be helpful
        # store the extra info.  Other plugin types have to raise the correct exception
        # such as AnsibleLookupError, so can't easily consume this.
        self.kwargs = kwargs or {}


def is_ansible_aws_error_code(code, e=None):
    """Check if the AnsibleAWSError exception is raised by a botocore exception with specific error code.

    Returns AnsibleAWSError if the error code matches, a dummy exception if it does not have an error code or does not match

    Example:
    try:
        describe_instances(connection, InstanceIds=['potato'])
    except is_ansible_aws_error_code('InvalidInstanceID.Malformed'):
        # handle the error for that code case
    except AnsibleAWSError as e:
        # handle the generic error case for all other codes
    """
    from botocore.exceptions import ClientError

    if e is None:
        import sys

        e = sys.exc_info()[1]
    if not isinstance(code, (list, tuple, set)):
        code = [code]
    if (
        isinstance(e, AnsibleAWSError)
        and e.exception
        and isinstance(e.exception, ClientError)
        and e.exception.response["Error"]["Code"] in code
    ):
        return AnsibleAWSError
    return type("NeverEverRaisedException", (Exception,), {})


def is_ansible_aws_error_message(msg, e=None):
    """Check if the AnsibleAWSError exception raised by a botocore exception contains a specific error message.

    Returns AnsibleAWSError if the error code matches, a dummy exception if it does not have an error code or does not match

    Example:
    try:
        describe_vpc_classic_link(connection, VpcIds=[vpc_id])
    except is_ansible_aws_error_message('The functionality you requested is not available in this region.'):
        # handle the error for that error message
    except AnsibleAWSError as e:
        # handle the generic error case for all other codes
    """
    from botocore.exceptions import ClientError

    if e is None:
        import sys

        e = sys.exc_info()[1]
    if (
        isinstance(e, AnsibleAWSError)
        and e.exception
        and isinstance(e.exception, ClientError)
        and msg in e.exception.response["Error"]["Message"]
    ):
        return AnsibleAWSError
    return type("NeverEverRaisedException", (Exception,), {})


class AnsibleBotocoreError(AnsibleAWSError):
    pass



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: sts_session_token
version_added: 1.0.0
short_description: obtain a session token from the AWS Security Token Service
description:
  - Obtain a session token from the AWS Security Token Service.
author:
  - Victor Costan (@pwnall)
options:
  duration_seconds:
    description:
      - The duration, in seconds, of the session token.
        See U(https://docs.aws.amazon.com/STS/latest/APIReference/API_GetSessionToken.html#API_GetSessionToken_RequestParameters)
        for acceptable and default values.
    type: int
  mfa_serial_number:
    description:
      - The identification number of the MFA device that is associated with the user who is making the GetSessionToken call.
    type: str
  mfa_token:
    description:
      - The value provided by the MFA device, if the trust policy of the user requires MFA.
    type: str
notes:
  - In order to use the session token in a following playbook task you must pass the I(access_key),
    I(secret_key) and I(session_token) parameters to modules that should use the session credentials.
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

RETURN = r"""
sts_creds:
    description: The Credentials object returned by the AWS Security Token Service
    returned: always
    type: list
    sample:
      access_key: ASIAXXXXXXXXXXXXXXXX
      expiration: "2016-04-08T11:59:47+00:00"
      secret_key: XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
      session_token: XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
changed:
    description: True if obtaining the credentials succeeds
    type: bool
    returned: always
"""


EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

# (more details: https://docs.aws.amazon.com/STS/latest/APIReference/API_GetSessionToken.html)
- name: Get a session token
  community.aws.sts_session_token:
    access_key: AKIA1EXAMPLE1EXAMPLE
    secret_key: 123456789abcdefghijklmnopqrstuvwxyzABCDE
    duration_seconds: 3600
  register: session_credentials

- name: Use the session token obtained above to tag an instance in account 123456789012
  amazon.aws.ec2_tag:
    access_key: "{{ session_credentials.sts_creds.access_key }}"
    secret_key: "{{ session_credentials.sts_creds.secret_key }}"
    session_token: "{{ session_credentials.sts_creds.session_token }}"
    resource: i-xyzxyz01
    state: present
    tags:
      MyNewTag: value
"""

try:
    import botocore
    from botocore.exceptions import ClientError
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


def normalize_credentials(credentials):
    access_key = credentials.get("AccessKeyId", None)
    secret_key = credentials.get("SecretAccessKey", None)
    session_token = credentials.get("SessionToken", None)
    expiration = credentials.get("Expiration", None)
    return {
        "access_key": access_key,
        "secret_key": secret_key,
        "session_token": session_token,
        "expiration": expiration,
    }


def get_session_token(connection, module):
    duration_seconds = module.params.get("duration_seconds")
    mfa_serial_number = module.params.get("mfa_serial_number")
    mfa_token = module.params.get("mfa_token")
    changed = False

    args = {}
    if duration_seconds is not None:
        args["DurationSeconds"] = duration_seconds
    if mfa_serial_number is not None:
        args["SerialNumber"] = mfa_serial_number
    if mfa_token is not None:
        args["TokenCode"] = mfa_token

    try:
        response = connection.get_session_token(**args)
        changed = True
    except ClientError as e:
        module.fail_json(msg=e)

    credentials = normalize_credentials(response.get("Credentials", {}))
    module.exit_json(changed=changed, sts_creds=credentials)


def main():
    argument_spec = dict(
        duration_seconds=dict(required=False, default=None, type="int"),
        mfa_serial_number=dict(required=False, default=None),
        mfa_token=dict(required=False, default=None, no_log=True),
    )

    module = AnsibleAWSModule(argument_spec=argument_spec)

    try:
        connection = module.client("sts")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS")

    get_session_token(connection, module)


if __name__ == "__main__":
    main()



# Copyright (c) 2018 Zim Kalinowski, <zikalino@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


try:
    from ansible.module_utils.ansible_release import __version__ as ANSIBLE_VERSION
except Exception:
    ANSIBLE_VERSION = 'unknown'

try:
    from azure.core._pipeline_client import PipelineClient
    from azure.core.polling import LROPoller
    from azure.core.pipeline import PipelineResponse
    from azure.core.pipeline.policies import BearerTokenCredentialPolicy, UserAgentPolicy
    from azure.mgmt.core.polling.arm_polling import ARMPolling
    import uuid
    from azure.core.configuration import Configuration
except ImportError:
    # This is handled in azure_rm_common
    Configuration = object

ANSIBLE_USER_AGENT = 'Ansible/{0}'.format(ANSIBLE_VERSION)


class GenericRestClientConfiguration(Configuration):

    def __init__(self, credential, subscription_id, credential_scopes=None, base_url=None):

        if credential is None:
            raise ValueError("Parameter 'credentials' must not be None.")
        if subscription_id is None:
            raise ValueError("Parameter 'subscription_id' must not be None.")
        if not base_url:
            base_url = 'https://management.azure.com'
        if not credential_scopes:
            credential_scopes = 'https://management.azure.com/.default'

        super(GenericRestClientConfiguration, self).__init__()

        self.user_agent_policy = UserAgentPolicy(ANSIBLE_USER_AGENT)
        self.credentials = credential
        self.subscription_id = subscription_id
        self.authentication_policy = BearerTokenCredentialPolicy(credential, credential_scopes)


class GenericRestClient(object):

    def __init__(self, credential, subscription_id, base_url=None, credential_scopes=None):
        self._config = GenericRestClientConfiguration(credential, subscription_id, credential_scopes[0])
        self._client = PipelineClient(base_url, config=self._config)
        self.models = None

        # Support http errors by status code
        self.error_map = {}

    def query(self, url, method, query_parameters, header_parameters, body, expected_status_codes, polling_timeout, polling_interval):
        # Construct and send request
        operation_config = {}

        request = None

        if header_parameters is None:
            header_parameters = {}

        header_parameters['x-ms-client-request-id'] = str(uuid.uuid1())

        if method == 'GET':
            request = self._client.get(url, query_parameters, header_parameters, body)
        elif method == 'PUT':
            request = self._client.put(url, query_parameters, header_parameters, body)
        elif method == 'POST':
            request = self._client.post(url, query_parameters, header_parameters, body)
        elif method == 'HEAD':
            request = self._client.head(url, query_parameters, header_parameters, body)
        elif method == 'PATCH':
            request = self._client.patch(url, query_parameters, header_parameters, body)
        elif method == 'DELETE':
            request = self._client.delete(url, query_parameters, header_parameters, body)
        elif method == 'MERGE':
            request = self._client.merge(url, query_parameters, header_parameters, body)

        response = self._client.send_request(request, **operation_config)

        if response.status_code not in expected_status_codes:
            return self.on_error(response)

        elif response.status_code == 202 and polling_timeout > 0:
            def get_long_running_output(response):
                return response
            poller = LROPoller(self._client,
                               PipelineResponse(None, response, None),
                               get_long_running_output,
                               ARMPolling(polling_interval, **operation_config))
            response = self.get_poller_result(poller, polling_timeout)

        return response

    def get_poller_result(self, poller, timeout):
        try:
            poller.wait(timeout=timeout)
            return poller.result()
        except Exception as exc:
            raise

    def on_error(self, response):
        """ handle errors in response
        """
        # raise common http errors
        error_type = self.error_map.get(response.status_code)
        if error_type:
            raise error_type(response=response)
        raise SendRequestException(response.text(), response.status_code)


class SendRequestException(Exception):
    def __init__(self, response, status_code):
        self.response = response
        self.status_code = status_code



#!/usr/bin/python
#
# Copyright (c) 2020 Fred-Sun, (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_registrationdefinition_info
version_added: '1.3.0'
short_description: Get RegistrationDefinition info
description:
    - Get info of RegistrationDefinition.
options:
    scope:
        description:
            - The subscription ID defines the subscription in which the registration definition will be retrieved.
            - If not specified, will use the subscription derived from AzureRMAuth.
        type: str
    registration_definition_id:
        description:
            - ID of the registration definition.
        type: str
extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags
author:
    - Fred-Sun (@Fred-Sun)
'''

EXAMPLES = '''
- name: Get Registration Definition
  azure_rm_registrationdefinition_info:
    registration_definition_id: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx

- name: Get All Registration Definitions from AzureRMAuth's subscription
  azure_rm_registrationdefinition_info:

- name: Get All Registration Definitions in the subscription levle
  azure_rm_registrationdefinition_info:
    scope: xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx
'''

RETURN = '''
registration_definitions:
    description:
        - A list of dict results where the key is the name of the RegistrationDefinition and the values are the facts for that RegistrationDefinition.
    returned: always
    type: complex
    contains:
        properties:
            description:
                - Properties of a registration definition.
            returned: always
            type: complex
            contains:
                description:
                    description:
                        - Description of the registration definition.
                    returned: always
                    type: str
                    sample: null
                authorizations:
                    description:
                        - Authorization tuple containing principal ID of the user/security group or service principal and id of the build-in role.
                    returned: always
                    type: complex
                    contains:
                        principal_id:
                            description:
                                - Principal ID of the security group/service principal/user that would be assigned permissions to the projected subscription.
                            returned: always
                            type: str
                            sample: null
                        role_definition_id:
                            description:
                                - The role definition identifier.
                                - The role will define all the permissions that the security group/service principal/user must have on the subscription.
                                - The role cannot be an owner role.
                            returned: always
                            type: str
                            sample: null
                registration_definition_name:
                    description:
                        - Name of the registration definition.
                    returned: always
                    type: str
                    sample: null
                managed_by_tenant_id:
                    description:
                        - ID of the managedBy tenant.
                    returned: always
                    type: str
                    sample: null
        plan:
            description:
                - Plan details for the managed services.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - The plan name.
                    returned: always
                    type: str
                    sample: null
                publisher:
                    description:
                        - The publisher ID.
                    returned: always
                    type: str
                    sample: null
                product:
                    description:
                        - The product code.
                    returned: always
                    type: str
                    sample: null
                version:
                    description:
                        - The plan's version.
                    returned: always
                    type: str
                    sample: null
        id:
            description:
                - Fully qualified path of the registration definition.
            returned: always
            type: str
            sample: null
        type:
            description:
                - Type of the resource.
            returned: always
            type: str
            sample: null
        name:
            description:
                - Name of the registration definition.
            returned: always
            type: str
            sample: null
'''

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ext import AzureRMModuleBase
try:
    from azure.mgmt.managedservices import ManagedServicesClient
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMRegistrationDefinitionInfo(AzureRMModuleBase):
    def __init__(self):
        self.module_arg_spec = dict(
            scope=dict(
                type='str'
            ),
            registration_definition_id=dict(
                type='str'
            )
        )

        self.scope = None
        self.registration_definition_id = None

        self.results = dict(changed=False)
        self.mgmt_client = None
        self.state = None
        self.url = None
        self.status_code = [200]

        self.mgmt_client = None
        super(AzureRMRegistrationDefinitionInfo, self).__init__(self.module_arg_spec, supports_check_mode=True, supports_tags=True)

    def exec_module(self, **kwargs):

        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        if not self.scope:
            self.scope = "/subscriptions/" + self.subscription_id
        else:
            self.scope = "/subscriptions/" + self.scope

        self.mgmt_client = self.get_mgmt_svc_client(ManagedServicesClient,
                                                    base_url=self._cloud_environment.endpoints.resource_manager,
                                                    api_version='2022-10-01',
                                                    suppress_subscription_id=True)

        if self.registration_definition_id is not None:
            self.results['registration_definitions'] = self.format_item(self.get())
        else:
            self.results['registration_definitions'] = self.format_item(self.list())
        return self.results

    def get(self):
        response = None

        try:
            response = self.mgmt_client.registration_definitions.get(scope=self.scope,
                                                                     registration_definition_id=self.registration_definition_id)
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def list(self):
        response = None

        try:
            response = self.mgmt_client.registration_definitions.list(scope=self.scope)
        except Exception as e:
            self.log('Could not get info for @(Model.ModuleOperationNameUpper).')

        return response

    def format_item(self, item):
        if hasattr(item, 'as_dict'):
            return [item.as_dict()]
        elif item is not None:
            result = []
            items = list(item)
            for tmp in items:
                result.append(tmp.as_dict())
            return result
        else:
            return []


def main():
    AzureRMRegistrationDefinitionInfo()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_image_pull

short_description: Pull Docker images from registries

version_added: 3.6.0

description:
  - Pulls a Docker image from a registry.
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: partial
    details:
      - When trying to pull an image with O(pull=always), the module assumes this is always changed in check mode.
      - When check mode is combined with diff mode, the pulled image's ID is always shown as V(unknown) in the diff.
  diff_mode:
    support: full
  idempotent:
    support: full

options:
  name:
    description:
      - Image name. Name format must be one of V(name), V(repository/name), or V(registry_server:port/name).
      - The name can optionally include the tag by appending V(:tag_name), or it can contain a digest by appending V(@hash:digest).
    type: str
    required: true
  tag:
    description:
      - Used to select an image when pulling. Defaults to V(latest).
      - If O(name) parameter format is C(name:tag) or C(image@hash:digest), then O(tag) will be ignored.
    type: str
    default: latest
  platform:
    description:
      - Ask for this specific platform when pulling.
    type: str
  pull:
    description:
      - Determines when to pull an image.
      - If V(always), will always pull the image.
      - If V(not_present), will only pull the image if no image of the name exists on the current Docker daemon, or if O(platform)
        does not match.
    type: str
    choices:
      - always
      - not_present
    default: always

requirements:
  - "Docker API >= 1.25"

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_image_pull
  - module: community.docker.docker_image_remove
  - module: community.docker.docker_image_tag
"""

EXAMPLES = r"""
---
- name: Pull an image
  community.docker.docker_image_pull:
    name: pacur/centos-7
    # Select platform for pulling. If not specified, will pull whatever docker prefers.
    platform: amd64
"""

RETURN = r"""
image:
  description: Image inspection results for the affected image.
  returned: success
  type: dict
  sample: {}
"""

import traceback
import typing as t

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._api.utils.utils import (
    parse_repository_tag,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._platform import (
    compare_platform_strings,
    compose_platform_string,
    normalize_platform_string,
)
from ansible_collections.community.docker.plugins.module_utils._util import (
    DockerBaseClass,
    is_image_name_id,
    is_valid_tag,
)


def image_info(image: dict[str, t.Any] | None) -> dict[str, t.Any]:
    result = {}
    if image:
        result["id"] = image["Id"]
    else:
        result["exists"] = False
    return result


class ImagePuller(DockerBaseClass):
    def __init__(self, client: AnsibleDockerClient) -> None:
        super().__init__()

        self.client = client
        self.check_mode = self.client.check_mode

        parameters = self.client.module.params
        self.name: str = parameters["name"]
        self.tag: str = parameters["tag"]
        self.platform: str | None = parameters["platform"]
        self.pull_mode: t.Literal["always", "not_present"] = parameters["pull"]

        if is_image_name_id(self.name):
            self.client.fail("Cannot pull an image by ID")
        if not is_valid_tag(self.tag, allow_empty=True):
            self.client.fail(f'"{self.tag}" is not a valid docker tag!')

        # If name contains a tag, it takes precedence over tag parameter.
        repo, repo_tag = parse_repository_tag(self.name)
        if repo_tag:
            self.name = repo
            self.tag = repo_tag

    def pull(self) -> dict[str, t.Any]:
        image = self.client.find_image(name=self.name, tag=self.tag)
        actions: list[str] = []
        diff = {"before": image_info(image), "after": image_info(image)}
        results = {
            "changed": False,
            "actions": actions,
            "image": image or {},
            "diff": diff,
        }

        if image and self.pull_mode == "not_present":
            if self.platform is None:
                return results
            host_info = self.client.info()
            wanted_platform = normalize_platform_string(
                self.platform,
                daemon_os=host_info.get("OSType"),
                daemon_arch=host_info.get("Architecture"),
            )
            image_platform = compose_platform_string(
                os=image.get("Os"),
                arch=image.get("Architecture"),
                variant=image.get("Variant"),
                daemon_os=host_info.get("OSType"),
                daemon_arch=host_info.get("Architecture"),
            )
            if compare_platform_strings(wanted_platform, image_platform):
                return results

        actions.append(f"Pulled image {self.name}:{self.tag}")
        if self.check_mode:
            results["changed"] = True
            diff["after"] = image_info({"Id": "unknown"})
        else:
            image, not_changed = self.client.pull_image(
                self.name, tag=self.tag, image_platform=self.platform
            )
            results["image"] = image
            results["changed"] = not not_changed
            diff["after"] = image_info(image)

        return results


def main() -> None:
    argument_spec = {
        "name": {"type": "str", "required": True},
        "tag": {"type": "str", "default": "latest"},
        "platform": {"type": "str"},
        "pull": {
            "type": "str",
            "choices": ["always", "not_present"],
            "default": "always",
        },
    }

    option_minimal_versions = {
        "platform": {"docker_api_version": "1.32"},
    }

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
        option_minimal_versions=option_minimal_versions,
    )

    try:
        results = ImagePuller(client).pull()
        client.module.exit_json(**results)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_firmware_policy
short_description: Firmware Policy configuration for Cisco Intersight
description:
  - Manages Firmware Policy configuration on Cisco Intersight.
  - A policy to configure firmware settings and versions for Cisco Intersight managed servers.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Profiles, Policies, and Pools that are created within a Custom Organization are applicable only to devices in the same Organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the Firmware Policy.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the Firmware Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
    default: []
  target_platform:
    description:
      - The platform type for which the firmware policy is intended.
      - Server type can be either Standalone or FIAttached.
      - This parameter is required when C(state) is C(present).
    type: str
    choices: ['Standalone', 'FIAttached']
  model_bundle_combo:
    description:
      - List of server model and firmware version pairs.
      - Each entry specifies which firmware version should be used for a specific server model family.
      - This parameter is required when C(state) is C(present).
    type: list
    elements: dict
    suboptions:
      model_family:
        description:
          - The server model family (e.g., UCSC-C220-M5, UCSC-C220-M4, UCSC-C220-M7).
        type: str
        required: true
      bundle_version:
        description:
          - The firmware bundle version to apply to the specified model family.
        type: str
        required: true
  exclude_drives:
    description:
      - When set to C(true), excludes local disk drives from firmware upgrades.
      - This adds local-disk to the ExcludeComponentList.
    type: bool
    default: false
  exclude_storage_controllers:
    description:
      - When set to C(true), excludes storage controllers from firmware upgrades.
      - This adds storage-controller to the ExcludeComponentList.
    type: bool
    default: false
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create a firmware policy for standalone servers
  cisco.intersight.intersight_firmware_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Standalone-Firmware-Policy"
    description: "Firmware policy for standalone servers"
    target_platform: "Standalone"
    model_bundle_combo:
      - model_family: "UCSC-C220-M5"
        bundle_version: "4.3(2.250037)"
      - model_family: "UCSC-C220-M4"
        bundle_version: "4.1(2m)"
    tags:
      - Key: "Environment"
        Value: "Production"

- name: Create a firmware policy with excluded components
  cisco.intersight.intersight_firmware_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Firmware-Policy-With-Exclusions"
    description: "Firmware policy excluding storage components"
    target_platform: "Standalone"
    model_bundle_combo:
      - model_family: "UCSC-C220-M7"
        bundle_version: "4.3(4.242038)"
    exclude_drives: true
    exclude_storage_controllers: true
    tags:
      - Key: "Site"
        Value: "Datacenter1"

- name: Delete a firmware policy
  cisco.intersight.intersight_firmware_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "Old-Firmware-Policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "Standalone-Firmware-Policy",
        "TargetPlatform": "Standalone",
        "ModelBundleCombo": [
            {
                "ModelFamily": "UCSC-C220-M5",
                "BundleVersion": "4.3(2.250037)"
            }
        ],
        "ExcludeComponentList": ["local-disk", "storage-controller"],
        "ObjectType": "firmware.Policy",
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_input(module):
    """
    Validate input parameters for the firmware policy module.
    """
    # Validate ModelBundleCombo entries
    model_bundle_combo = module.params.get('model_bundle_combo')
    for combo in model_bundle_combo:
        if not combo.get('model_family'):
            module.fail_json(msg="model_family is required for each entry in model_bundle_combo")
        if not combo.get('bundle_version'):
            module.fail_json(msg="bundle_version is required for each entry in model_bundle_combo")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict', default=[]),
        target_platform=dict(type='str', choices=['Standalone', 'FIAttached']),
        exclude_drives=dict(type='bool', default=False),
        exclude_storage_controllers=dict(type='bool', default=False),
        model_bundle_combo=dict(
            type='list',
            elements='dict',
            options=dict(
                model_family=dict(type='str', required=True),
                bundle_version=dict(type='str', required=True)
            ),
        ),
    )

    required_if = [
        ['state', 'present', ['target_platform', 'model_bundle_combo']],
    ]

    module = AnsibleModule(
        argument_spec,
        required_if=required_if,
        supports_check_mode=True,
    )
    if module.params['state'] == 'present':
        validate_input(module)
    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Resource path used to configure policy
    resource_path = '/firmware/Policies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    if module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['TargetPlatform'] = intersight.module.params['target_platform']

        # Set ModelBundleCombo
        model_bundle_combo = []
        for combo in module.params['model_bundle_combo']:
            model_bundle_combo.append({
                'ModelFamily': combo['model_family'],
                'BundleVersion': combo['bundle_version']
            })
        intersight.api_body['ModelBundleCombo'] = model_bundle_combo

        # Set exclude list
        exclude_components = []
        if module.params['exclude_drives']:
            exclude_components.append('local-disk')
        if module.params['exclude_storage_controllers']:
            exclude_components.append('storage-controller')
        intersight.api_body['ExcludeComponentList'] = exclude_components

    intersight.configure_policy_or_profile(resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_ldap_policy
short_description: Manage LDAP Policies for Cisco Intersight
description:
  - Create, update, and delete LDAP Policies on Cisco Intersight.
  - Manage LDAP groups and providers associated with LDAP policies.
  - LDAP policies enable authentication of Cisco IMC users using an LDAP server.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/iam/LdapPolicy/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the LDAP Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the LDAP Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  enable_ldap:
    description:
      - If enabled, LDAP server performs authentication.
    type: bool
    default: true
  base_dn:
    description:
      - Base Distinguished Name (DN).
      - Starting point from where server will search for users and groups.
      - Required when state is present.
    type: str
  domain:
    description:
      - The IPv4 domain that all users must be in.
      - Required when state is present.
    type: str
  timeout:
    description:
      - LDAP authentication timeout duration, in seconds.
      - Valid range is 0-180.
    type: int
    default: 0
  enable_encryption:
    description:
      - If enabled, the endpoint encrypts all information it sends to the LDAP server.
    type: bool
    default: false
  bind_method:
    description:
      - Authentication method to access LDAP servers.
      - C(logincredentials) uses the user credentials entered at login.
      - C(anonymous) uses no credentials to access the LDAP server.
      - C(configuredcredentials) uses a specific set of credentials configured for the LDAP server.
    type: str
    choices: [logincredentials, anonymous, configuredcredentials]
    default: logincredentials
  bind_dn:
    description:
      - Distinguished Name (DN) of the user, that is used to authenticate against LDAP servers.
      - Required when bind_method is configuredcredentials.
    type: str
  password:
    description:
      - The password of the user for initial bind process.
      - Can have any character except spaces, tabs, line breaks.
      - Cannot be more than 254 characters.
      - Required when bind_method is configuredcredentials.
    type: str
  filter:
    description:
      - Criteria to identify entries in search requests.
      - Required when state is present.
    type: str
  group_attribute:
    description:
      - Groups to which an LDAP entry belongs.
      - Required when state is present.
    type: str
  attribute:
    description:
      - Role and locale information of the user.
      - Required when state is present.
    type: str
  group_authorization:
    description:
      - If enabled, user authorization is also done at the group level for LDAP users not in the local user database.
    type: bool
    default: false
  nested_group_search:
    description:
      - If enabled, an extended search walks the chain of ancestry all the way to the root.
      - Returns all the groups and subgroups, each of those groups belong to recursively.
    type: bool
    default: false
  nested_group_search_depth:
    description:
      - Search depth to look for a nested LDAP group in an LDAP group map.
      - Valid range is 1-128.
      - Only applicable when nested_group_search is true.
    type: int
    default: 128
  enable_dns:
    description:
      - Enables DNS to access LDAP servers.
      - When enabled, LDAP providers cannot be specified (DNS discovery is used instead).
      - When disabled, at least one LDAP provider must be configured.
    type: bool
    default: false
  dns_source:
    description:
      - Source of the domain name used for the DNS SRV request.
      - C(extracted) extracts the domain name from the login ID entered by the user.
      - C(configured) uses the configured search domain.
      - C(configuredextracted) uses configured search domain first, then extracted.
      - Only applicable when enable_dns is true.
    type: str
    choices: [extracted, configured, configuredextracted]
    default: extracted
  search_domain:
    description:
      - Domain name that acts as a source for a DNS query.
      - Required when dns_source is configured or configuredextracted.
    type: str
  search_forest:
    description:
      - Forest name that acts as a source for a DNS query.
      - Required when dns_source is configured or configuredextracted.
    type: str
  user_search_precedence:
    description:
      - Search precedence between local user database and LDAP user database.
      - C(localuserdb) searches local user database first.
      - C(ldapuserdb) searches LDAP user database first.
    type: str
    choices: [localuserdb, ldapuserdb]
    default: localuserdb
  ldap_groups:
    description:
      - List of LDAP groups to be created and attached to the LDAP policy.
      - Each group defines the mapping between LDAP server groups and Intersight roles.
    type: list
    elements: dict
    suboptions:
      name:
        description:
          - LDAP Group name in the LDAP server database.
        type: str
        required: true
      state:
        description:
          - Whether to create/update or delete the LDAP group.
        type: str
        choices: [present, absent]
        default: present
      group_dn:
        description:
          - LDAP Group DN in the LDAP server database.
          - Required when state is present.
        type: str
      domain:
        description:
          - LDAP server domain the Group resides in.
        type: str
      role:
        description:
          - Role assigned to all users in this LDAP server group.
          - C(admin) provides full administrative access.
          - C(readonly) provides read-only access.
          - C(user) provides standard user access.
          - Only 'admin' role is supported in domain.
          - Required when state is present.
        type: str
        choices: [admin, readonly, user]
        default: admin
  ldap_providers:
    description:
      - List of LDAP providers (servers) to be created and attached to the LDAP policy.
      - Providers define the LDAP servers to connect to.
      - Cannot be specified when enable_dns is true (DNS discovery is used instead).
      - Required when enable_dns is false (at least one provider must be specified).
    type: list
    elements: dict
    suboptions:
      server:
        description:
          - IP address or hostname of the LDAP server.
          - Required when state is present.
        type: str
        required: true
      state:
        description:
          - Whether to create/update or delete the LDAP provider.
        type: str
        choices: [present, absent]
        default: present
      port:
        description:
          - Port number on which the LDAP server is listening.
        type: int
        default: 389
      vendor:
        description:
          - Type of LDAP server.
          - C(openldap) for OpenLDAP servers.
          - C(msad) for Microsoft Active Directory.
        type: str
        choices: [openldap, msad]
        default: openldap
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create LDAP Policy with DNS enabled
  cisco.intersight.intersight_ldap_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "ldap-dns-policy"
    description: "LDAP policy using DNS for server discovery"
    enable_ldap: true
    base_dn: "dc=example,dc=com"
    domain: "example.com"
    timeout: 30
    enable_encryption: true
    bind_method: "logincredentials"
    filter: "sAMAccountName"
    group_attribute: "memberOf"
    attribute: "CiscoAvPair"
    group_authorization: true
    nested_group_search: true
    nested_group_search_depth: 64
    enable_dns: true
    dns_source: "configured"
    search_domain: "example.com"
    search_forest: "example.com"
    user_search_precedence: "localuserdb"
    ldap_groups:
      - name: "admin-group"
        group_dn: "cn=admins,ou=groups,dc=example,dc=com"
        domain: "example.com"
        role: "admin"
      - name: "readonly-group"
        group_dn: "cn=readers,ou=groups,dc=example,dc=com"
        role: "readonly"
    tags:
      - Key: "Environment"
        Value: "Production"
    state: present

- name: Create LDAP Policy with providers
  cisco.intersight.intersight_ldap_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "ldap-provider-policy"
    description: "LDAP policy with specific servers"
    enable_ldap: true
    base_dn: "company"
    domain: "company.local"
    timeout: 0
    enable_encryption: false
    bind_method: "anonymous"
    filter: "sAMAccountName"
    group_attribute: "memberOf"
    attribute: "CiscoAvPair"
    group_authorization: false
    nested_group_search: false
    enable_dns: false
    user_search_precedence: "localuserdb"
    ldap_providers:
      - server: "10.10.10.10"
        port: 389
        vendor: "openldap"
      - server: "10.10.10.11"
        port: 389
        vendor: "openldap"
    ldap_groups:
      - name: "users-group"
        group_dn: "company"
        role: "user"
    state: present

- name: Create LDAP Policy with configured credentials
  cisco.intersight.intersight_ldap_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "ldap-configured-creds"
    description: "LDAP policy with configured bind credentials"
    enable_ldap: true
    base_dn: "dc=corp,dc=net"
    domain: "corp.net"
    timeout: 60
    enable_encryption: true
    bind_method: "configuredcredentials"
    bind_dn: "cn=admin,dc=corp,dc=net"
    password: "SecurePassword123"
    filter: "uid"
    group_attribute: "gidNumber"
    attribute: "description"
    group_authorization: true
    nested_group_search: true
    nested_group_search_depth: 128
    enable_dns: false
    user_search_precedence: "ldapuserdb"
    ldap_providers:
      - server: "ldap.corp.net"
        port: 636
        vendor: "msad"
    ldap_groups:
      - name: "administrators"
        group_dn: "cn=administrators,ou=groups,dc=corp,dc=net"
        domain: "corp.net"
        role: "admin"
    state: present

- name: Update LDAP Policy - manage group states
  cisco.intersight.intersight_ldap_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "ldap-dns-policy"
    description: "Updated LDAP policy"
    enable_ldap: true
    base_dn: "dc=example,dc=com"
    domain: "example.com"
    filter: "sAMAccountName"
    group_attribute: "memberOf"
    attribute: "CiscoAvPair"
    enable_dns: true
    ldap_groups:
      - name: "admin-group"
        group_dn: "cn=admins,ou=groups,dc=example,dc=com"
        role: "admin"
        state: present
      - name: "old-group"
        state: absent
      - name: "new-group"
        group_dn: "cn=new,ou=groups,dc=example,dc=com"
        role: "user"
        state: present
    state: present

- name: Delete LDAP Policy
  cisco.intersight.intersight_ldap_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "old-ldap-policy"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "ldap-dns-policy",
        "ObjectType": "iam.LdapPolicy",
        "Enabled": true,
        "BaseProperties": {
            "BaseDn": "dc=example,dc=com",
            "Domain": "example.com",
            "Timeout": 30,
            "EnableEncryption": true,
            "BindMethod": "LoginCredentials",
            "Filter": "sAMAccountName",
            "GroupAttribute": "memberOf",
            "Attribute": "CiscoAvPair",
            "EnableGroupAuthorization": true,
            "EnableNestedGroupSearch": true,
            "NestedGroupSearchDepth": 64
        },
        "EnableDns": true,
        "DnsParameters": {
            "Source": "Configured",
            "SearchDomain": "example.com",
            "SearchForest": "example.com"
        },
        "UserSearchPrecedence": "LocalUserDb",
        "Groups": [
            {
                "Name": "admin-group",
                "GroupDn": "cn=admins,ou=groups,dc=example,dc=com",
                "Domain": "example.com"
            }
        ],
        "Providers": []
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_input(module):
    """
    Validate module input parameters.
    """
    if module.params['state'] == 'present':
        required_fields = ['base_dn', 'domain', 'filter', 'group_attribute', 'attribute']
        for field in required_fields:
            if not module.params.get(field):
                module.fail_json(msg=f"{field} is required when state is 'present'")
        if module.params['bind_method'] == 'configuredcredentials':
            if not module.params.get('bind_dn'):
                module.fail_json(msg="bind_dn is required when bind_method is 'configuredcredentials'")
            if not module.params.get('password'):
                module.fail_json(msg="password is required when bind_method is 'configuredcredentials'")
        if module.params['enable_dns']:
            if module.params['dns_source'] in ['configured', 'configuredextracted']:
                if not module.params.get('search_domain'):
                    module.fail_json(msg="search_domain is required when dns_source is 'configured' or 'configuredextracted'")
                if not module.params.get('search_forest'):
                    module.fail_json(msg="search_forest is required when dns_source is 'configured' or 'configuredextracted'")
        if module.params['timeout'] < 0 or module.params['timeout'] > 180:
            module.fail_json(msg="timeout must be between 0 and 180 seconds")
        if module.params['nested_group_search']:
            depth = module.params['nested_group_search_depth']
            if depth < 1 or depth > 128:
                module.fail_json(msg="nested_group_search_depth must be between 1 and 128")

        ldap_groups = module.params.get('ldap_groups') or []
        for group in ldap_groups:
            if group.get('state', 'present') == 'present':
                if not group.get('group_dn'):
                    module.fail_json(msg=f"group_dn is required for LDAP group '{group['name']}' when state is 'present'")
                if not group.get('role'):
                    module.fail_json(msg=f"role is required for LDAP group '{group['name']}' when state is 'present'")
        if module.params['enable_dns']:
            if module.params.get('ldap_providers'):
                module.fail_json(msg="ldap_providers cannot be specified when enable_dns is true. DNS discovery is used instead of explicit providers.")
        else:
            ldap_providers = module.params.get('ldap_providers') or []
            if not ldap_providers:
                module.fail_json(msg="ldap_providers is required when enable_dns is false. Specify at least one LDAP server.")
            for provider in ldap_providers:
                if provider.get('state', 'present') == 'present':
                    if not provider.get('server'):
                        module.fail_json(msg="server is required for LDAP provider when state is 'present'")


def get_endpoint_role_moid(intersight, role_name):
    """
    Get EndPointRole MOID by role name.
    """
    intersight.get_resource(
        resource_path='/iam/EndPointRoles',
        query_params={
            '$filter': f"Name eq '{role_name}' and Type eq 'IMC'",
            '$select': 'Moid'
        }
    )
    return intersight.result['api_response'].get('Moid')


def build_ldap_group_api_body(intersight, module, group_config, ldap_policy_moid):
    """
    Build LDAP group API body.
    """
    role_name = group_config['role']
    endpoint_role_moid = get_endpoint_role_moid(intersight, role_name)
    if not endpoint_role_moid:
        module.fail_json(msg=f"EndPointRole '{role_name}' not found")
    group_api_body = {
        'Name': group_config['name'],
        'GroupDn': group_config['group_dn'],
        'EndPointRole': [endpoint_role_moid],
        'LdapPolicy': ldap_policy_moid
    }
    if group_config.get('domain'):
        group_api_body['Domain'] = group_config['domain']
    return group_api_body


def build_ldap_provider_api_body(provider_config, ldap_policy_moid):
    """
    Build LDAP provider API body.
    """
    vendor_map = {
        'openldap': 'OpenLDAP',
        'msad': 'MSAD'
    }
    provider_api_body = {
        'Server': provider_config['server'],
        'Port': provider_config.get('port', 389),
        'Vendor': vendor_map[provider_config.get('vendor', 'openldap')],
        'LdapPolicy': ldap_policy_moid
    }
    return provider_api_body


def main():
    ldap_group_options = dict(
        name=dict(type='str', required=True),
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        group_dn=dict(type='str'),
        domain=dict(type='str'),
        role=dict(type='str', choices=['admin', 'readonly', 'user'], default='admin')
    )
    ldap_provider_options = dict(
        server=dict(type='str', required=True),
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        port=dict(type='int', default=389),
        vendor=dict(type='str', choices=['openldap', 'msad'], default='openldap')
    )
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        enable_ldap=dict(type='bool', default=True),
        base_dn=dict(type='str'),
        domain=dict(type='str'),
        timeout=dict(type='int', default=0),
        enable_encryption=dict(type='bool', default=False),
        bind_method=dict(type='str', choices=['logincredentials', 'anonymous', 'configuredcredentials'], default='logincredentials'),
        bind_dn=dict(type='str'),
        password=dict(type='str', no_log=True),
        filter=dict(type='str'),
        group_attribute=dict(type='str'),
        attribute=dict(type='str'),
        group_authorization=dict(type='bool', default=False),
        nested_group_search=dict(type='bool', default=False),
        nested_group_search_depth=dict(type='int', default=128),
        enable_dns=dict(type='bool', default=False),
        dns_source=dict(type='str', choices=['extracted', 'configured', 'configuredextracted'], default='extracted'),
        search_domain=dict(type='str'),
        search_forest=dict(type='str'),
        user_search_precedence=dict(type='str', choices=['localuserdb', 'ldapuserdb'], default='localuserdb'),
        ldap_groups=dict(type='list', elements='dict', options=ldap_group_options),
        ldap_providers=dict(type='list', elements='dict', options=ldap_provider_options)
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    if module.params['state'] == 'present':
        validate_input(module)

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''
    resource_path = '/iam/LdapPolicies'

    intersight.api_body = {
        'Organization': {
            'Name': intersight.module.params['organization'],
        },
        'Name': intersight.module.params['name']
    }

    # Start propogating the api body for the main resource (Ldap Policy)
    if intersight.module.params['state'] == 'present':
        intersight.set_tags_and_description()
        intersight.api_body['Enabled'] = intersight.module.params['enable_ldap']

        bind_method_map = {
            'logincredentials': 'LoginCredentials',
            'anonymous': 'Anonymous',
            'configuredcredentials': 'ConfiguredCredentials'
        }

        base_properties = {
            'BaseDn': intersight.module.params['base_dn'],
            'Domain': intersight.module.params['domain'],
            'Timeout': intersight.module.params['timeout'],
            'EnableEncryption': intersight.module.params['enable_encryption'],
            'BindMethod': bind_method_map[intersight.module.params['bind_method']],
            'Filter': intersight.module.params['filter'],
            'GroupAttribute': intersight.module.params['group_attribute'],
            'Attribute': intersight.module.params['attribute'],
            'EnableGroupAuthorization': intersight.module.params['group_authorization'],
            'EnableNestedGroupSearch': intersight.module.params['nested_group_search']
        }

        if intersight.module.params['bind_method'] == 'configuredcredentials':
            base_properties['BindDn'] = intersight.module.params['bind_dn']
            base_properties['Password'] = intersight.module.params['password']

        if intersight.module.params['nested_group_search']:
            base_properties['NestedGroupSearchDepth'] = intersight.module.params['nested_group_search_depth']

        intersight.api_body['BaseProperties'] = base_properties
        intersight.api_body['EnableDns'] = intersight.module.params['enable_dns']
        if intersight.module.params['enable_dns']:
            dns_source_map = {
                'extracted': 'Extracted',
                'configured': 'Configured',
                'configuredextracted': 'ConfiguredExtracted'
            }
            dns_parameters = {
                'Source': dns_source_map[intersight.module.params['dns_source']]
            }
            if intersight.module.params['dns_source'] in ['configured', 'configuredextracted']:
                dns_parameters['SearchDomain'] = intersight.module.params['search_domain']
                dns_parameters['SearchForest'] = intersight.module.params['search_forest']

            intersight.api_body['DnsParameters'] = dns_parameters

        user_search_precedence_map = {
            'localuserdb': 'LocalUserDb',
            'ldapuserdb': 'LDAPUserDb'
        }
        intersight.api_body['UserSearchPrecedence'] = user_search_precedence_map[intersight.module.params['user_search_precedence']]

    intersight.configure_policy_or_profile(resource_path=resource_path)

    # Save the LDAP policy response
    ldap_policy_response = intersight.result['api_response']
    ldap_policy_moid = None

    if intersight.module.params['state'] == 'present' and ldap_policy_response:
        ldap_policy_moid = ldap_policy_response.get('Moid')

    # Process LDAP groups
    groups_response = []
    ldap_groups = intersight.module.params.get('ldap_groups') or []
    if intersight.module.params['state'] == 'present' and ldap_groups:
        for group_config in ldap_groups:
            group_state = group_config.get('state', 'present')
            if group_state == 'present':
                group_api_body = build_ldap_group_api_body(intersight, module, group_config, ldap_policy_moid)
                intersight.api_body = group_api_body

            resource_path = '/iam/LdapGroups'
            custom_filter = f"Name eq '{group_config['name']}' and LdapPolicy.Moid eq '{ldap_policy_moid}'"
            intersight.configure_secondary_resource(
                resource_path=resource_path,
                state=group_state,
                custom_filter=custom_filter
            )
            if group_state == 'present':
                groups_response.append(intersight.result['api_response'])

    # Process LDAP providers
    providers_response = []
    ldap_providers = intersight.module.params.get('ldap_providers') or []
    if intersight.module.params['state'] == 'present' and not intersight.module.params['enable_dns']:
        if ldap_providers:
            for provider_config in ldap_providers:
                provider_state = provider_config.get('state', 'present')
                if provider_state == 'present':
                    provider_api_body = build_ldap_provider_api_body(provider_config, ldap_policy_moid)
                    intersight.api_body = provider_api_body

                resource_path = '/iam/LdapProviders'
                custom_filter = f"Server eq '{provider_config['server']}' and LdapPolicy.Moid eq '{ldap_policy_moid}'"
                intersight.configure_secondary_resource(
                    resource_path=resource_path,
                    state=provider_state,
                    custom_filter=custom_filter
                )
                if provider_state == 'present':
                    providers_response.append(intersight.result['api_response'])

    if ldap_policy_response:
        ldap_policy_response['Groups'] = groups_response
        ldap_policy_response['Providers'] = providers_response
        intersight.result['api_response'] = ldap_policy_response
    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_online_endpoint_info
version_added: "3.17.0"
short_description: List ML Online Endpoint resources
description:
    - List ML Online Endpoint resources.
options:
    name:
        description:
            - Name of the Azure ML online endpoint.
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    include_auth_keys:
        description:
            - If True then auth_keys will be populated in the return data.
        default: false
        type: bool
    local:
        description:
            - List all local endpoints.
        type: bool
        default: false

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Get Specific ML Online Endpoint by name w/auth_keys
  azure.azcollection.azure_rm_ml_online_endpoint_info:
    name: endpoint-command-xxxxxxxxxx
    include_auth_keys: true
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace

- name: List Local ML Online Endpoints
  azure.azcollection.azure_rm_ml_online_endpoint_info:
    resource_group: myResourceGroup
    ml_workspace: myMLWorkspace
    local: true
'''

RETURN = '''
ml_online_endpoints:
    description:
        - Online Endpoints that match the query.
    returned: always
    type: dict
    sample: [
      {
          "auth_mode": "key",
          "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-online_endpoint/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/onlineEndpoints/endpoint-command-xxxxxxxxxx",
          "identity": {
              "principal_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "tenant_id": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "type": "system_assigned"
          },
          "auth_keys": {
              "primary_key": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
              "secondary_key": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
          },
          "kind": "Managed",
          "location": "eastus",
          "mirror_traffic": {},
          "name": "endpoint-command-xxxxxxxxxx",
          "openapi_uri": "https://endpoint-command-xxxxxxxxxx.eastus.inference.ml.azure.com/swagger.json",
          "properties": {
              "AzureAsyncOperationUri": "https://management.azure.com/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/providers/Microsoft.MachineLearningServices/locations/eastus/mfeOperationsStatus/oeidp:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx:xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx?api-version=2022-02-01-preview",
              "azureml.onlineendpointid": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/xxxxx-ml-online_endpoint/providers/microsoft.machinelearningservices/workspaces/workspace-xxxxxxxxxx/onlineendpoints/endpoint-command-xxxxxxxxxx",
              "createdAt": "2026-04-09T15:04:48.831114+0000",
              "createdBy": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
              "lastModifiedAt": "2026-04-09T15:04:48.831114+0000"
          },
          "provisioning_state": "Succeeded",
          "public_network_access": "enabled",
          "scoring_uri": "https://endpoint-command-xxxxxxxxxx.eastus.inference.ml.azure.com/score",
          "tags": {},
          "traffic": {}
      }
    ]
'''  # NOQA


try:
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLOnlineEndpointInfo(MLClientCommon):
    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            include_auth_keys=dict(
                type='bool',
                default=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            local=dict(
                type='bool',
                default=False,
            ),
        )

        self._client = None
        self.ml_registry = None

        self.results = dict(
            ml_online_endpoints=[]
        )

        mutually_exclusive = [('include_auth_keys', 'local')]

        super(AzureRMMLOnlineEndpointInfo, self).__init__(self.module_arg_spec,
                                                          supports_tags=False,
                                                          supports_check_mode=True,
                                                          facts_module=True,
                                                          mutually_exclusive=mutually_exclusive,
                                                          )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name:
            ml_online_endpoints = [self.get(self.name,
                                            include_auth_keys=self.include_auth_keys,
                                            local=self.local)]
        else:
            results = self.client.online_endpoints.list(local=self.local)
            ml_online_endpoints = [self.get(x.name, include_auth_keys=self.include_auth_keys, local=self.local) for x in results]

        self.results['ml_online_endpoints'] = ml_online_endpoints
        return self.results

    def get(self, name, include_auth_keys=False, local=False):
        try:
            result = self.client.online_endpoints.get(name=name,
                                                      local=self.local)
            ml_online_endpoint = self.entity_to_dict(result)
            if include_auth_keys and not local:
                auth_keys = self.client.online_endpoints.get_keys(name=name)
                ml_online_endpoint['auth_keys'] = dict(primary_key=auth_keys.primary_key,
                                                       secondary_key=auth_keys.secondary_key)
        except ResourceNotFoundError:
            ml_online_endpoint = None

        return ml_online_endpoint


def main():
    AzureRMMLOnlineEndpointInfo()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright 2020 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Ospfv3 parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


def _tmplt_ospf_vrf_cmd(process):
    command = "router ospfv3"
    vrf = "{vrf}".format(**process)
    if "vrf" in process and vrf != "default":
        command += " vrf " + vrf
    return command


def _tmplt_ospf_address_family_cmd(config_data):
    afi = "{afi}".format(**config_data)
    if afi == "router":
        command = ""
    else:
        command = "address-family " + afi
    return command


def _tmplt_ospf_adjacency_cmd(config_data):
    command = "adjacency exchange-start threshold"
    if "adjacency" in config_data:
        command += " {threshold}".format(**config_data["adjacency"]["exchange_start"])
    return command


def _tmplt_ospf_auto_cost(config_data):
    if "auto_cost" in config_data:
        command = "auto-cost"
        if "reference_bandwidth" in config_data["auto_cost"]:
            command += " reference-bandwidth " + config_data["auto_cost"]["reference_bandwidth"]
        return command


def _tmplt_ospf_area_authentication(config_data):
    if "area_id" in config_data:
        command = "area {area_id} authentication ipsec spi ".format(**config_data)
        command += "{spi} {algorithm}".format(**config_data["authentication"])
        if "passphrase" in config_data["authentication"]:
            command += " passphrase"
        if (
            "encrypt_key" in config_data["authentication"]
            and config_data["authentication"]["encrypt_key"] is False
        ):
            command += " 0"
        if (
            "hidden_key" in config_data["authentication"]
            and config_data["authentication"]["hidden_key"] is True
        ):
            command += " 7"
        if "passphrase" not in config_data["authentication"]:
            command += " {key}".format(**config_data["authentication"])
        else:
            command += " {passphrase}".format(**config_data["authentication"])
        return command


def _tmplt_ospf_area_encryption(config_data):
    if "area_id" in config_data:
        command = "area {area_id} encryption ipsec spi ".format(**config_data)
        command += "{spi} esp {encryption} {algorithm}".format(**config_data["encryption"])
        if "passphrase" in config_data["encryption"]:
            command += " passphrase"
        if (
            "encrypt_key" in config_data["encryption"]
            and config_data["encryption"]["encrypt_key"] is False
        ):
            command += " 0"
        if (
            "hidden_key" in config_data["encryption"]
            and config_data["encryption"]["hidden_key"] is True
        ):
            command += " 7"
        if "passphrase" not in config_data["encryption"]:
            command += " {key}".format(**config_data["encryption"])
        else:
            command += " {passphrase}".format(**config_data["encryption"])
        return command


def _tmplt_ospf_area_nssa(config_data):
    if "nssa" in config_data:
        command = "area {area_id} nssa".format(**config_data)
        if "default_information_originate" in config_data["nssa"]:
            command += " default-information-originate"
            if "metric" in config_data["nssa"]["default_information_originate"]:
                command += " metric {metric}".format(
                    **config_data["nssa"]["default_information_originate"],
                )
            if "metric_type" in config_data["nssa"]["default_information_originate"]:
                command += " metric-type {metric_type}".format(
                    **config_data["nssa"]["default_information_originate"],
                )
            if "nssa_only" in config_data["nssa"]["default_information_originate"]:
                command += " nssa-only"
        if config_data["nssa"].get("nssa_only"):
            command += " nssa-only"
        if config_data["nssa"].get("translate"):
            command += " translate type7 always"
        if config_data["nssa"].get("no_summary"):
            command += " no-summary"
        return command


def _tmplt_ospf_area_range(config_data):
    if "area_id" in config_data:
        command = "area {area_id} range".format(**config_data)
        if "address" in config_data:
            command += " {address}".format(**config_data)
        if "subnet_address" in config_data:
            command += " {subnet_address}".format(**config_data)
        if "subnet_mask" in config_data:
            command += " {subnet_mask}".format(**config_data)
        if "advertise" in config_data:
            if config_data.get("advertise"):
                command += " advertise"
            else:
                command += " not-advertise"
        if "cost" in config_data:
            command += " cost {cost}".format(**config_data)
        return command


def _tmplt_ospf_area_stub(config_data):
    if "stub" in config_data:
        command = "area {area_id} stub".format(**config_data)
        if "summary_lsa" in config_data["stub"]:
            if not config_data["stub"]["summary_lsa"]:
                command += " no-summary"
        return command


def _tmplt_ospf_default_information(config_data):
    if "default_information" in config_data:
        command = "default-information"
        if "originate" in config_data["default_information"]:
            command += " originate"
        if "always" in config_data["default_information"]:
            command += " always"
        if "metric" in config_data["default_information"]:
            command += " metric {metric}".format(**config_data["default_information"])
        if "metric_type" in config_data["default_information"]:
            command += " metric-type {metric_type}".format(**config_data["default_information"])
        if "route_map" in config_data["default_information"]:
            command += " route-map {route_map}".format(**config_data["default_information"])
        return command


def _tmplt_ospf_log_adjacency_changes(config_data):
    if "log_adjacency_changes" in config_data:
        command = "log-adjacency-changes"
        if "detail" in config_data["log_adjacency_changes"]:
            if config_data["log_adjacency_changes"].get("detail"):
                command += " detail"
        return command


def _tmplt_ospf_max_metric(config_data):
    if "max_metric" in config_data:
        command = "max-metric"
        if "router_lsa" in config_data["max_metric"]:
            command += " router-lsa"
        if "external_lsa" in config_data["max_metric"]["router_lsa"]:
            command += " external-lsa"
            if "max_metric_value" in config_data["max_metric"]["router_lsa"]["external_lsa"]:
                command += " {max_metric_value}".format(
                    **config_data["max_metric"]["router_lsa"]["external_lsa"],
                )
        if "include_stub" in config_data["max_metric"]["router_lsa"]:
            if config_data["max_metric"]["router_lsa"].get("include_stub"):
                command += " include-stub"
        if "on_startup" in config_data["max_metric"]["router_lsa"]:
            command += " on-startup {wait_period}".format(
                **config_data["max_metric"]["router_lsa"]["on_startup"],
            )
        if "summary_lsa" in config_data["max_metric"]["router_lsa"]:
            command += " summary-lsa"
            if "max_metric_value" in config_data["max_metric"]["router_lsa"]["summary_lsa"]:
                command += " {max_metric_value}".format(
                    **config_data["max_metric"]["router_lsa"]["summary_lsa"],
                )
        return command


def _tmplt_ospf_redistribute(config_data):
    command = "redistribute {routes}".format(**config_data)
    if "route_map" in config_data:
        command += " route-map {route_map}".format(**config_data)
    return command


def _tmplt_ospf_timers_lsa(config_data):
    command = ""
    if "lsa" in config_data["timers"]:
        command += "timers lsa {direction}".format(**config_data["timers"]["lsa"])
        if config_data["timers"]["lsa"]["direction"] == "rx":
            command += " min interval "
        else:
            command += " delay initial "
        if config_data["timers"]["lsa"].get("initial"):
            command += str(config_data["timers"]["lsa"]["initial"])
        if config_data["timers"]["lsa"].get("min"):
            command += str(config_data["timers"]["lsa"]["min"])
        if config_data["timers"]["lsa"].get("max"):
            command += str(config_data["timers"]["lsa"]["max"])
    return command


def _tmplt_ospf_timers_spf(config_data):
    command = ""
    if "spf" in config_data["timers"]:
        command += "timers spf delay initial "
        command += "{initial} {min} {max}".format(**config_data["timers"]["spf"])
    return command


def _tmplt_ospf_bfd(config_data):
    if os_version < "4.23":
        command = "bfd all-interfaces"
    else:
        command = "bfd default"
    return command


os_version = "4.23"


class Ospfv3Template(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        global os_version
        super(Ospfv3Template, self).__init__(
            lines=lines,
            tmplt=self,
            module=module,
        )
        if self._connection:
            os_version = self._get_os_version()

    def _get_os_version(self):
        os_version = self._connection.get_device_info()["network_os_version"]
        return os_version

    PARSERS = [
        {
            "name": "vrf",
            "getval": re.compile(
                r"""
                ^router\s
                ospfv3
                \svrf
                \s(?P<vrf>\S+)
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_vrf_cmd,
            "result": {"processes": {"vrf": "{{ vrf }}"}},
            "shared": True,
        },
        {
            "name": "vrf",
            "getval": re.compile(
                r"""
                ^router\s
                ospfv3
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_vrf_cmd,
            "result": {"processes": {"vrf": '{{ "default" }}'}},
            "shared": True,
        },
        {
            "name": "address_family",
            "getval": re.compile(
                r"""
                \s*address-family
                \s(?P<afi>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_address_family_cmd,
            "compval": "address_family",
            "result": {
                "processes": {
                    "address_family": {"{{ afi }}": {"afi": "{{ afi }}"}},
                },
            },
            "shared": True,
        },
        {
            "name": "adjacency",
            "getval": re.compile(
                r"""
                \s*adjacency
                \s+exchange-start
                \s+threshold
                \s+(?P<threshold>\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_adjacency_cmd,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "adjacency": {
                                "exchange_start": {
                                    "threshold": "{{ threshold|int }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "auto_cost",
            "getval": re.compile(
                r"""\s+(?P<auto_cost>auto-cost)*
                    \s*(?P<ref_band>reference-bandwidth\s\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_auto_cost,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "auto_cost": {
                                "reference_bandwidth": '{{ ref_band.split(" ")[1] }}',
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.default_cost",
            "getval": re.compile(
                r"""\s+area
                    \s(?P<area_id>\S+)*
                    \sdefault-cost*
                    \s*(?P<default_cost>\S+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "area {{ area_id }} default-cost {{ default_cost }}",
            "compval": "default_cost",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "areas": {
                                "{{ area_id }}": {
                                    "area_id": "{{ area_id }}",
                                    "default_cost": "{{ default_cost|int }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.authentication",
            "getval": re.compile(
                r"""
                \s*area
                \s+(?P<area_id>\S+)
                \s+authentication
                \s+ipsec
                \s+spi
                \s+(?P<val>\d+)
                \s+(?P<algorithm>md5|sha1)
                \s*(?P<passphrase>passphrase)*
                \s*(?P<type>0|7)*
                \s*(?P<line>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_authentication,
            "compval": "authentication",
            "remval": "area {{ area_id }} authentication",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "areas": {
                                "{{ area_id }}": {
                                    "area_id": "{{ area_id }}",
                                    "authentication": {
                                        "spi": "{{ val }}",
                                        "algorithm": "{{ algorithm }}",
                                        "encrypt_key": '{{ False if type is defined and type == "0" }}',
                                        "hidden_key": '{{ True if type is defined and type == "7" }}',
                                        "passphrase": "{{ line if passphrase is defined }}",
                                        "key": "{{ str(line) if passphrase is undefined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.encryption",
            "getval": re.compile(
                r"""
                \s*area
                \s+(?P<area_id>\S+)*
                \s+encryption
                \s+ipsec
                \s+spi
                \s+(?P<val>\d+)
                \s+esp
                \s*(?P<encryption>\S+)
                \s*(?P<algorithm>md5|sha1)*
                \s*(?P<passphrase>passphrase)*
                \s*(?P<type>0|7)*
                \s*(?P<line>\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_encryption,
            "compval": "encryption",
            "remval": "area {{ area_id }} encryption",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "areas": {
                                "{{ area_id }}": {
                                    "area_id": "{{ area_id }}",
                                    "encryption": {
                                        "spi": "{{ val }}",
                                        "encryption": "{{ encryption }}",
                                        "algorithm": "{{ algorithm }}",
                                        "encrypt_key": "{{ False if type is defined and type == '0'}}",
                                        "passphrase": "{{line if passphrase is defined }}",
                                        "hidden_key": "{{ True if type is defined and type == '7'}}",
                                        "key": "{{ line if passphrase is not defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.nssa",
            "getval": re.compile(
                r"""
                \s+area\s(?P<area_id>\S+)
                \s(?P<nssa>nssa)
                \s*(?P<def_origin>default-information-originate)*
                \s*(metric)*
                \s*(?P<metric>\d+)*
                \s*(metric-type)*
                \s*(?P<metric_type>\d+)*
                \s*(?P<no_summary>no-summary)*
                \s*(?P<translate>translate.*)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_nssa,
            "compval": "nssa",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "areas": {
                                "{{ area_id }}": {
                                    "area_id": "{{ area_id }}",
                                    "nssa": {
                                        "set": "{{ True if nssa is defined and def_origin is undefined and "
                                        "no_summary is undefined and translate is undefined }}",
                                        "default_information_originate": {
                                            "set": "{{ True if def_origin is defined and metric is undefined and "
                                            "metric_type is undefined and nssa_only is undefined }}",
                                            "metric": "{{ metric.split(" ")[1]|int }}",
                                            "metric_type": "{{ metric_type.split(" ")[1]|int }}",
                                            "nssa_only": "{{ True if nssa_only is defined }}",
                                        },
                                        "translate": "{{ True if translate is defined }}",
                                        "no_summary": "{{ True if no_summary is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.ranges",
            "getval": re.compile(
                r"""
                \s*area
                \s+(?P<area_id>\S+)*
                \s+range
                \s+(?P<address>\S+)*
                \s*(?P<not_advertise>not-advertise)*
                \s*(?P<subnet_mask>\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})*
                \s*(?P<subnet_address>\S+)*
                \s*(?P<cost>cost)*
                \s*(?P<cost_val>\d+)
                *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_range,
            "compval": "ranges",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "areas": {
                                "{{ area_id }}": {
                                    "area_id": "{{ area_id }}",
                                    "ranges": [
                                        {
                                            "address": "{{ address }}",
                                            "subnet_mask": "{{ subnet_mask }}",
                                            "advertise": "{{ not not_advertise }}",
                                            "cost": "{{ cost_val }}",
                                        },
                                    ],
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "area.stub",
            "getval": re.compile(
                r"""\s+area\s(?P<area_id>\S+)
                    \s(?P<stub>stub)*
                    \s*(?P<no_sum>no-summary)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_area_stub,
            "compval": "stub",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "areas": {
                                "{{ area_id }}": {
                                    "area_id": "{{ area_id }}",
                                    "stub": {
                                        "set": "{{ True if stub is defined and no_sum is undefined }}",
                                        "summary_lsa": "{{ True if no_sum is defined }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""\s+bfd*
                    \s*(?P<bfd>all-interfaces)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_bfd,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "bfd": {
                                "all_interfaces": "{{ True if bfd is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "bfd",
            "getval": re.compile(
                r"""\s+bfd*
                    \s*(?P<bfd>default)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_bfd,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "bfd": {
                                "all_interfaces": "{{ True if bfd is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "default_information",
            "getval": re.compile(
                r"""\s+default-information*
                    \s*(?P<originate>originate)*
                    \s*(?P<always>always)*
                    \s*(?P<metric>metric\s\d+)*
                    \s*(?P<metric_type>metric-type\s\d+)*
                    \s*(?P<route_map>route-map\s\S+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_default_information,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "default_information": {
                                "originate": "{{ True if originate is defined }}",
                                "always": "{{ True if always is defined }}",
                                "metric": "{{ metric.split(" ")[1]|int }}",
                                "metric_type": "{{ metric_type.split(" ")[1]|int }}",
                                "route_map": "{{ route_map.split(" ")[1] }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "default_metric",
            "getval": re.compile(
                r"""\s+default-metric(?P<default_metric>\s\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "default-metric {{ default_metric }}",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "default_metric": "{{ default_metric| int}}",
                        },
                    },
                },
            },
        },
        {
            "name": "distance",
            "getval": re.compile(
                r"""\s+distance
                    \s+ospf
                    \s+intra-area
                    \s+(?P<distance>\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "distance ospf intra-area {{ distance }}",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "distance": "{{ distance| int}}",
                        },
                    },
                },
            },
        },
        {
            "name": "fips_restrictions",
            "getval": re.compile(
                r"""
                \s+(?P<fips>fips\s*\S+)
                *$""",
                re.VERBOSE,
            ),
            "setval": "fips restrictions",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "fips_restrictions": "{{ True if fips is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "graceful_restart_period",
            "getval": re.compile(
                r"""
                \s+graceful-restart
                \s*grace-period*
                \s*(?P<period>\d+)*
                $""",
                re.VERBOSE,
            ),
            "setval": "graceful-restart grace-period {{ graceful_restart.grace_period }}",
            "remval": "graceful-restart",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "graceful_restart": {
                                "grace_period": "{{ period|int }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "graceful_restart",
            "getval": re.compile(
                r"""
                \s+graceful-restart
                $""",
                re.VERBOSE,
            ),
            "setval": "graceful-restart",
            "remval": "graceful-restart",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "graceful_restart": {"set": "{{ True }}"},
                        },
                    },
                },
            },
        },
        {
            "name": "graceful_restart_helper",
            "getval": re.compile(
                r"""\s+(?P<grace>graceful-restart-helper)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "{{ grace }}",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "graceful_restart_helper": {
                                "set": "{{ True if grace is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "log_adjacency_changes",
            "getval": re.compile(
                r"""\s+(?P<log>log-adjacency-changes)*
                    \s*(?P<detail>detail)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_log_adjacency_changes,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "log_adjacency_changes": {
                                "set": "{{ True if log is defined and detail is undefined }}",
                                "detail": "{{ True if detail is defined }}",
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "max_metric",
            "getval": re.compile(
                r"""\s+max-metric
                    \s+(?P<router_lsa>router-lsa)
                    \s*(?P<external_lsa>external-lsa)*
                    \s*(?P<external_lsa_metric>\d+)*
                    \s*(?P<on_startup>on-startup)*
                    \s*(?P<wait_for_bgp>wait-for-bgp)*
                    \s*(?P<startup_time>\d+)*
                    \s*(?P<summary_lsa>summary-lsa)*
                    \s*(?P<summary_lsa_metric>\d+)*
                    \s*(?P<include_stub>include-stub)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_max_metric,
            "remval": "max-metric router-lsa",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "max_metric": {
                                "router_lsa": {
                                    "set": "{{ True if router_lsa is defined and external_lsa is undefined and "
                                    "include_stub is undefined and on_startup is undefined and "
                                    "summary_lsa is undefined }}",
                                    "external_lsa": {
                                        "set": "{{ True if external_lsa is defined and external_lsa_metric is undefined }}",
                                        "max_metric_value": "{{ external_lsa_metric }}",
                                    },
                                    "include_stub": "{{ True if include_stub is defined }}",
                                    "on_startup": {
                                        "wait_period": "{{ startup_time }}",
                                        "wait_for_bgp": "{{ True if wait_for_bgp is defined }}",
                                    },
                                    "summary_lsa": {
                                        "set": "{{ True if summary_lsa is defined and summary_lsa_metric is undefined }}",
                                        "max_metric_value": "{{ summary_lsa_metric }}",
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "maximum_paths",
            "getval": re.compile(
                r"""\s+maximum-paths*
                    \s+(?P<paths>\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "maximum-paths {{ maximum_paths }}",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "maximum_paths": "{{ paths }}",
                        },
                    },
                },
            },
        },
        {
            "name": "passive_interface",
            "getval": re.compile(
                r"""
                \s*(?P<passive>passive-interface.*)
                 *$""",
                re.VERBOSE,
            ),
            "setval": "passive-interface default",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "passive_interface": "{{ True if passive is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "redistribute",
            "getval": re.compile(
                r"""
                \s+redistribute
                \s(?P<route>\S+)
                \s*(?P<rmpa>route-map)*
                \s*(?P<map>\S+)*
                $""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_redistribute,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "redistribute": [
                                {
                                    "routes": "{{ route }}",
                                    "route_map": "{{ map }}",
                                },
                            ],
                        },
                    },
                },
            },
        },
        {
            "name": "router_id",
            "getval": re.compile(
                r"""
                \s+router-id
                \s(?P<id>\S+)$""",
                re.VERBOSE,
            ),
            "setval": ("router-id" " {{ router_id }}"),
            "remval": "router-id",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "router_id": "{{ id }}",
                        },
                    },
                },
            },
        },
        {
            "name": "shutdown",
            "getval": re.compile(
                r"""\s+(?P<shutdown>shutdown)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "shutdown",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "shutdown": "{{ True if shutdown is defined }}",
                        },
                    },
                },
            },
        },
        {
            "name": "timers.out_delay",
            "getval": re.compile(
                r"""\s+timers
                    \sout-delay
                    \s(?P<out_delay>\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "timers out-delay {{ timers.out_delay }}",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "timers": {"out_delay": "{{ out_delay }}"},
                        },
                    },
                },
            },
        },
        {
            "name": "timers.pacing",
            "getval": re.compile(
                r"""\s+timers
                    \spacing
                    \sflood
                    \s(?P<pacing>\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": "timers pacing flood {{ timers.pacing }}",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "timers": {"pacing": "{{ pacing }}"},
                        },
                    },
                },
            },
        },
        {
            "name": "timers.lsa",
            "getval": re.compile(
                r"""\s+timers
                    \s(?P<lsa>lsa)
                    \s(?P<dir>rx|tx)
                    \s*(min\sinterval)*
                    \s*(delay\sinitial)*
                    \s*(?P<initial>\d+)*
                    \s*(?P<min>\d+)*
                    \s*(?P<max>\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_timers_lsa,
            "compval": "timers.lsa",
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "timers": {
                                "lsa": {
                                    "direction": "{{ dir }}",
                                    "initial": "{{ initial }}",
                                    "min": "{{ min_delay }}",
                                    "max": "{{ max_delay }}",
                                },
                            },
                        },
                    },
                },
            },
        },
        {
            "name": "timers.spf",
            "getval": re.compile(
                r"""\s+timers
                    \s+(?P<spf>spf)
                    \s+(delay\sinitial)
                    \s*(?P<initial>\d+)
                    \s*(?P<min>\d+)
                    \s*(?P<max>\d+)
                    *$""",
                re.VERBOSE,
            ),
            "setval": _tmplt_ospf_timers_spf,
            "result": {
                "processes": {
                    "address_family": {
                        '{{ afi|default("router", true) }}': {
                            "timers": {
                                "spf": {
                                    "initial": "{{ initial }}",
                                    "min": "{{ min }}",
                                    "max": "{{ max }}",
                                },
                            },
                        },
                    },
                },
            },
        },
    ]
